//*******************************************************
//
//	NTPclock
//
//	7SEG LED
//
//	2015.9.20	KAWAKAMI Yukio
//*******************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "hw_config.h"

#include "main.h"
#include "port.h"
#include "config.h"
#include "com.h"
#include "timer.h"
#include "rtc.h"
#include "ssprintf.h"
#include "log.h"
#include "led.h"

volatile uint8_t LedBuff[6];
volatile uint8_t LedBuffP;

const uint8_t LedFont[16] = {
	0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x27, 0x7F, 0x6F,
	0x77, 0x7C, 0x39, 0x7E, 0x79, 0x71
};

//************************************************************
//	TIM2 interrupt
//************************************************************
void TIM2_IRQHandler(void){

	if (TIM_GetFlagStatus(TIM2, TIM_FLAG_Update) == SET){
		// 割り込みフラグのクリア
		TIM_ClearITPendingBit(TIM2, TIM_IT_Update);

		SEG_OUT(0);
		if (++LedBuffP >= sizeof(LedBuff))	LedBuffP = 0;

		SEG_OUT(LedBuff[LedBuffP]);
		switch(LedBuffP){
		  case 0:	GPIO_SET_HIGH(GPIOB, PIN_LED1);	break;		
		  case 1:	GPIO_SET_HIGH(GPIOB, PIN_LED2);	break;		
		  case 2:	GPIO_SET_HIGH(GPIOB, PIN_LED3);	break;		
		  case 3:	GPIO_SET_HIGH(GPIOB, PIN_LED5);	break;		
		  case 4:	GPIO_SET_HIGH(GPIOB, PIN_LED6);	break;		
		  case 5:	GPIO_SET_HIGH(GPIOB, PIN_LED7);	break;		
		}
	}
}


//*******************************************************
//	7SEGに時間表示
//*******************************************************
void print_time(struct RTC_TIME *rtc){
	char	msg[8];
	int		i;

	ssprintf(msg, "%2d%02d%02d",
			 rtc->hour, rtc->min, rtc->sec);

	for (i = 0; i < 6; i++){
		if (msg[i] == ' '){
			LedBuff[i] = 0;
		} else {
			LedBuff[i] = LedFont[msg[i] & 0x0F];
		}
	}
	LedBuff[1] |= 0x80;
	LedBuff[3] |= 0x80;
}


//*******************************************************
//	7SEG LED 初期化
//*******************************************************
void init_7seg(void){
	RCC_ClocksTypeDef	RCC_Clocks;
	NVIC_InitTypeDef	NVIC_InitStructure;
	TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;

	// システムクロック読み込み
	RCC_GetClocksFreq(&RCC_Clocks);

	// タイマ2設定
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
	TIM_TimeBaseStructInit(&TIM_TimeBaseStructure); 
	TIM_TimeBaseStructure.TIM_Period = 25 - 1;	// 2.5ms
	TIM_TimeBaseStructure.TIM_Prescaler = RCC_Clocks.SYSCLK_Frequency / 10000 - 1; // 10kHz
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

	// NVIC割り込み設定
	NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 4;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
	TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE); // 割り込み許可
	// タイマ動作開始
	TIM_Cmd(TIM2, ENABLE);
}
