//*******************************************************
//
//	NTPclock
//
//	ログの記録・読み出し
//
//	2015.11.15	KAWAKAMI Yukio
//*******************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "hw_config.h"
#include "main.h"
#include "port.h"
#include "config.h"
#include "com.h"
#include "ssprintf.h"
#include "rtc.h"
#include "rom.h"
#include "log.h"

extern COM_WORK	STDIO_COM;
extern struct RTC_TIME	Rtc;
extern uint32_t NowSec;		// 現在秒数値

uint32_t	WriteLogP;

//*******************************************************
//	指定時間以降のログ出力
//*******************************************************
void print_logs(uint32_t tm){
	uint32_t	adrs = 0;
	LOG_DATA	log;

	while((adrs = read_log(adrs, &log)) != LOG_ADRS_SEARCH_END){
		if (log.time >= tm)	break;
	}

	while(adrs && (adrs != LOG_ADRS_SEARCH_END)){
		struct RTC_TIME	rtc;
		char	msg[28];
		WDR;
		seccount_to_rtctime(log.time, &rtc);
		ssprintf(msg, "20%02d/%02d/%02d %02d:%02d:%02d ",
				 rtc.year, rtc.month, rtc.day,
				 rtc.hour, rtc.min, rtc.sec);
		cputs(msg);
		cputs(log.msg);
		cputs("\n");
		wait_send_com(&STDIO_COM);
		adrs = read_log(adrs, &log);
	}
}


//*******************************************************
//	ログ記録＆シリアル出力
//*******************************************************
void write_log_msg(char *msg){

	write_log(msg);
	cputs(msg);
	if (strchr(msg, '\n') == NULL){
		cputs("\n");
	}
}


//*******************************************************
//	ログ記録
//*******************************************************
void write_log(char *msg){
	LOG_DATA	data;
	char	*p;

	if (WriteLogP == 0){
		uint32_t	adrs;
		// 初めてなので記録開始位置を調べる
		for (adrs = LOG_START;
			 adrs < (LOG_START + LOG_SIZE);
			 adrs += sizeof(LOG_DATA)){
			ROM_read(adrs, (uint8_t *)&data, sizeof(LOG_DATA));
			if (data.time != 0xFFFFFFFF)	break;
		}
		if (adrs >= (LOG_START + LOG_SIZE)){
			// まっさらだった
			WriteLogP = LOG_START;
		} else {
			// 一部記録済み
			for (;
				 adrs < (LOG_START + LOG_SIZE);
				 adrs += sizeof(LOG_DATA)){
				ROM_read(adrs, (uint8_t *)&data, sizeof(LOG_DATA));
				if (data.time == 0xFFFFFFFF)	break;
			}
			if (adrs < (LOG_START + LOG_SIZE)){
				WriteLogP = adrs;
			} else {
				WriteLogP = LOG_START;
			}
		}
	}

	data.time = NowSec;
	memset(data.msg, 0, 28);
	strncpy(data.msg, msg, 28-1);
	p = strchr(data.msg, '\r');
	if (p != NULL)	*p = 0;
	p = strchr(data.msg, '\n');
	if (p != NULL)	*p = 0;
	ROM_write(WriteLogP, (uint8_t *)&data, sizeof(LOG_DATA));
	WriteLogP += sizeof(LOG_DATA);
	if (WriteLogP >= (LOG_START + LOG_SIZE)){
		WriteLogP = LOG_START;
	}
	if ((WriteLogP & ~ROM_PAGE_MASK) == 0){
		// ページ消去
		ROM_page_erase(WriteLogP);
	}
}


//*******************************************************
//	ログ１件読み出し
//	adrs = 0に設定すると最初の１件を読みだし
//	戻り値は次のアドレス。0なら無し
//*******************************************************
uint32_t read_log(uint32_t adrs, LOG_DATA *log){
	LOG_DATA	data;

	if (adrs >= (LOG_START + LOG_SIZE))	return LOG_ADRS_SEARCH_END;

	if (adrs == LOG_ADRS_SEARCH_START){
		// 初めてなので記録開始位置を調べる
		for (adrs = LOG_START;
			 adrs < (LOG_START + LOG_SIZE);
			 adrs += sizeof(LOG_DATA)){
			ROM_read(adrs, (uint8_t *)&data, sizeof(LOG_DATA));
			if (data.time == 0xFFFFFFFF)	break;
		}
		if (adrs >= (LOG_START + LOG_SIZE)){
			// データで埋まっていた(本来はありえない)
			adrs = LOG_START;
		} else {
			// 一部記録済み
			for (;
				 adrs < (LOG_START + LOG_SIZE);
				 adrs += sizeof(LOG_DATA)){
				ROM_read(adrs, (uint8_t *)&data, sizeof(LOG_DATA));
				if (data.time != 0xFFFFFFFF)	break;
			}
			if (adrs >= (LOG_START + LOG_SIZE)){
				adrs = LOG_START;
			}
		}
	}

	ROM_read(adrs, (uint8_t *)&data, sizeof(LOG_DATA));
	adrs += sizeof(LOG_DATA);
	if (adrs >= (LOG_START + LOG_SIZE)){
		adrs = LOG_START;
	}
	if (data.time == 0xFFFFFFFF){
		return LOG_ADRS_SEARCH_END;
	} else {
		*log = data;
		return adrs;
	}
}


//*******************************************************
//	ログ全消去
//*******************************************************
void clear_log(void){
	uint32_t	adrs;

	for (adrs = LOG_START;
		 adrs < (LOG_START + LOG_SIZE);
		 adrs += sizeof(LOG_DATA)){
		WDR;
		ROM_page_erase(adrs);
	}

	WriteLogP = LOG_START;
}
