//*******************************************************
//
//	NTPclock
//
//	Application Main
//
//	2015.12.4	KAWAKAMI Yukio
//*******************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "hw_config.h"

#include "main.h"
#include "port.h"
#include "config.h"
#include "com.h"
#include "timer.h"
#include "rtc.h"
#include "program.h"
#include "ssprintf.h"
#include "term.h"
#include "spi.h"
#include "wifi.h"
#include "log.h"
#include "led.h"

uint8_t	Debug = 0;
CONFIG	Config;

extern COM_WORK		STDIO_COM;

int ProgramFlag = 0;

const char *BootMsg = "NTPclock Version " VERSION "\n"
	"Build Date : " __DATE__ " " __TIME__ "\n";

extern volatile uint8_t		RtcIntFlag;
extern volatile uint8_t		RtcAlarmFlag;
extern volatile uint32_t	TickCount;

uint8_t SecFlag;	// 秒更新された
extern struct RTC_TIME	Rtc;	// 現在時刻
uint32_t NowSec;		// 現在秒数値
uint32_t SecCount = 0;	// 起動後経過時間(秒)

/*******************************************************************************
* Function Name  : main.
* Description    : Main routine.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
int main(void){
	uint32_t	csr;

	csr = RCC->CSR;
	RCC->CSR |= (1<<24);	// リセットフラグクリア

	__disable_irq();	// 割り込み禁止

	SCB->VTOR = NVIC_VectTab_FLASH | (APPLICATION_ADDRESS & 0x1FFFFF80);

	init_ports();

	__enable_irq();

	wait_ms(200);

	LED_ON;

	// 独立型ウォッチドッグタイマー
	IWDG->KR = 0x5555;
	IWDG->PR = 0x06;		// 1/256
	IWDG->RLR = 480;		// ウォッチドッグ 約3秒
	WDR;
	IWDG->KR = 0xCCCC;		// enable

	if (read_backup_ram(BKP_REG_BACKUP) != RTC_BACKUP_MARK){
		init_RTC();
		write_backup_ram(BKP_REG_BACKUP, RTC_BACKUP_MARK);
	}
	set_RTC_int();	// RTC 1秒割り込み設定

	//RCC_ClearFlag();

	load_config(&Config);
	if (Config.mark != CONFIG_MARK){
		init_config(&Config);
		save_config(&Config);
	}
	if (Config.flags & CONFIG_FLAGS_DEBUG)	Debug = 1;

	{
		char	msg[32];

		init_com(&STDIO_COM, USART1, 115200);
		cputs(BootMsg);

		read_RTC(&Rtc);
		NowSec = get_seccount(&Rtc);
		ssprintf(msg, "20%02d/%02d/%02d ", Rtc.year, Rtc.month, Rtc.day);
		cputs(msg);
		ssprintf(msg, "%02d:%02d:%02d\n", Rtc.hour, Rtc.min, Rtc.sec);
		cputs(msg);
	}
	if (csr & (1<<31)){
		write_log_msg("Low-power Reset");
	}
	if (csr & (1<<30)){
		write_log_msg("WWDG Reset");
	}
	if (csr & (1<<29)){
		write_log_msg("IWDG Reset");
	}

	write_log("Start System " VERSION);

	init_7seg();
	init_wifi();

	LED_OFF;
	for(;;){
		WDR;
		SLEEP;

		SecFlag = 0;
		if (RtcIntFlag){
			RtcIntFlag = 0;
			SecFlag = 1;
			SecCount++;		// 他からの変更禁止
			// RTC現在時刻読み込み
			read_RTC(&Rtc);
			NowSec = get_seccount(&Rtc);
			print_time(&Rtc);

			if (((Rtc.hour == 4)||(Rtc.hour == 16))&&
				(Rtc.min == 51)&&(Rtc.sec == 39)){
				start_wifi();	// Wifi接続・NTP時刻取得開始
			}
		}

		job_wifi();
		job_term();
	}
}


//*******************************************************
//	1秒ごとに割り込み実行する処理
//*******************************************************
void job_sec_intr(void){

}


//*******************************************************
//	時計合わせ
//*******************************************************
void adjust_time(uint32_t now){
	char	msg[32];
	uint32_t	last_adjust, tm;
	long	ppm, diff, inter;

	tm = NowSec;
	if (tm == now){
		write_log_msg("Time is Adjustment-free");
		return;
	}

	last_adjust = read_backup_ram(BKP_REG_ADJUSTED_TIME_L);
	last_adjust |= ((uint32_t)read_backup_ram(
						BKP_REG_ADJUSTED_TIME_H)) << 16;

	write_RTC_value(now);
	write_backup_ram(BKP_DR1, RTC_BACKUP_MARK);
	if (tm > now){
		diff = (tm - now);
		diff *= -1;
	} else {
		diff = now - tm;
	}
	inter = now - last_adjust;
	ppm = diff;
	ppm *= 100 * 10000;
	ppm += inter / 2;
	ppm /= inter;
	ssprintf(msg, "%+dSec / %dSec %+dppm Adjusted OK\n", diff, inter, ppm);
	cputs(msg);

	// ログ記録
	if (abs(diff) <= 3600){
		ssprintf(msg, "Set Time %+d/%d %+dppm", diff, inter, ppm);
		write_log(msg);
	} else {
		write_log("Set Time");
	}
}


//********************************************************
//	ブートローダ起動
//********************************************************
void boot(void){
	//typedef void (*func)(void);
	//func	jump_to_app;
	u32		*jump_adrs;
	u32		*p = (u32 *)BOOTLOADER_ADDRESS;
	u32		sp;

	sp = *p++;
	jump_adrs = (u32 *)(*p);
	//jump_to_app = (func)jump_adrs;

	if (((u32)jump_adrs > (u32)BOOTLOADER_ADDRESS)&&
		((u32)jump_adrs < (u32)APPLICATION_ADDRESS)&&
		(sp > RAM_ADDRESS)&&
		(sp <= (RAM_ADDRESS + RAM_SIZE))){
		char	*p;
		int		i;

		__disable_irq();	// 割り込み禁止

		for (i = 0; i < 8; i++){
			NVIC->ICER[i] = NVIC->IABR[i];
		}

		SysTick->CTRL = 0;
		EXTI_DeInit();
		TIM_DeInit(TIM1);
		TIM_DeInit(TIM2);
		TIM_DeInit(TIM3);
		TIM_DeInit(TIM4);
		RTC_ITConfig(RTC_IT_SEC, DISABLE);
		USART_DeInit(USART1);
		USART_DeInit(USART2);
		USART_DeInit(USART3);

		// RAM末尾にブートマーク書き込み。これに備えてspは4つずらしてある
		p = (char *)(RAM_ADDRESS + RAM_SIZE - 4);
		strncpy(p, "BOOT", 4);

		SCB->VTOR = NVIC_VectTab_FLASH | (BOOTLOADER_ADDRESS & 0x1FFFFF80);

		IWDG->KR = 0x5555;
		IWDG->PR = 0x06;		// 1/256
		IWDG->RLR = 16;		// ウォッチドッグ 約0.1秒
		WDR;
		IWDG->KR = 0xCCCC;		// enable

		for(;;){
			NOP;	// WDR
		}

		//__set_MSP(sp);
		//jump_to_app();
	}
}


//*******************************************************
//	GPIOポート初期化
//*******************************************************
void init_ports(void){
	GPIO_InitTypeDef GPIO_InitStructure;
	//RCC_ClocksTypeDef RCC_Clocks;
	//NVIC_InitTypeDef NVIC_InitStructure;
	//TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;

	// PWRとBKPクロック供給	SLEEPモードに必要らしい
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
	//RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
	//RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE);

	// JTAGピンを GPIOにリマップ。これ以降、JTAG I/Fは無効
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);
	GPIO_PinRemapConfig(GPIO_Remap_SWJ_Disable, ENABLE);
	BKP_TamperPinCmd(DISABLE);	// タンパーピン無効

	// GPIO初期値
	LED_OFF;
	MCS_OFF;
	RST_OFF;
	SEG_OUT(0);

	// GPIO出力設定
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Pin = PIN_M_CS | PIN_LED;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;	// ノイズを避ける
	GPIO_InitStructure.GPIO_Pin = 0x00FF;	// 7SEG アノード側
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	// GPIO オープンドレイン
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_OD;
	GPIO_InitStructure.GPIO_Pin = PIN_RST;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = 0xFF00;	// 7SEG カソード側
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	// GPIO入力プルアップ
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_InitStructure.GPIO_Pin = PIN_WUP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// GPIO入力プルダウン
	//GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;

	// GPIO入力
	//GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;;
	
	init_timer();
	spi_init(SPI1, SPI_BaudRatePrescaler_2);	// 12MHz
	spi_trx(SPI1, 0xFF);
}


#ifdef USE_FULL_ASSERT
/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert_param error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert_param error line source number
* Output         : None
* Return         : None
*******************************************************************************/
void assert_failed(uint8_t* file, uint32_t line){
	char	msg[32];
	cputs("assert_failed:");
	cputs((char *)file);
	ssprintf(msg, " %d\n", line);
	cputs(msg);
	wait_send_com(&STDIO_COM);
}
#endif
