//*******************************************************
//
//	NTPclock
//
//	NTP処理
//
//	2015.11.27	KAWAKAMI Yukio
//*******************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "hw_config.h"
#include "main.h"
#include "config.h"
#include "port.h"
#include "com.h"
#include "ssprintf.h"
#include "wifi.h"
#include "rtc.h"
#include "log.h"
#include "ntp.h"

extern COM_WORK	STDIO_COM;
extern COM_WORK	WIFI_COM;
extern CONFIG	Config;
extern uint8_t	Debug;

extern uint32_t	SecCount;	// 起動後の経過秒数
extern struct RTC_TIME	Rtc;
extern uint32_t NowSec;		// 現在秒数値

extern uint16_t	WifiStat;

uint8_t	*NtpP = NULL;
uint8_t	NtpN;

//*******************************************************
//	uitn32_t endian change
//*******************************************************
static uint32_t swap32(uint32_t n){
	uint8_t	*p;
	p = (uint8_t *)&n;
	return (p[0] << 24)|(p[1] << 16)|(p[2] << 8)|p[3];
}

#define	htonl(x)	swap32(x)
#define ntohl(x)	swap32(x)


//*******************************************************
//	NTP起動
//*******************************************************
int start_ntp(void){

	if ((WifiStat & 0xF000) == WIFI_STAT_NTP){
		return 2;
	} else if ((WifiStat == WIFI_STAT_WAIT)||
			   (WifiStat == WIFI_STAT_START)){
		WifiStat = WIFI_STAT_NTP;
		return 1;
	} else {
		return 0;
	}
}


//*******************************************************
//	NTP処理
//*******************************************************
uint16_t wifi_ntp(uint16_t stat, int res){
	static uint32_t	timeout = 0;
	struct NTP_Packet	ntp;
	uint32_t	last_adjust, diff;

	switch(stat & 0x00FF){
	  case 0:
		last_adjust = read_backup_ram(BKP_REG_ADJUSTED_TIME_L);
		last_adjust |= ((uint32_t)read_backup_ram(
							BKP_REG_ADJUSTED_TIME_H)) << 16;
		if (NowSec >= last_adjust){
			diff = NowSec - last_adjust;
		} else {
			diff = last_adjust - NowSec;
		}
		if (diff <= 5){	// 補正直後
			timeout = SecCount + 1;
			stat = WIFI_STAT_SLEEP;		// SLEEP
			break;
		}

		cputs("Start NTP\n");
		timeout = SecCount + 5;
		res = 0;
		send_wifi_command("AT+CIPMUX=1\r\n", 13);
		stat++;
		break;

	  case 1:
		if (res == WIFI_RES_OK){
			char	msg[64];
			ssprintf(msg, "AT+CIPSTART=4,\"UDP\",\"%s\",123\r\n",
					 Config.ntp_server);
			send_wifi_command(msg, strlen(msg));
			stat++;
		}
		break;

	  case 2:
		if (res == WIFI_RES_OK){
			wifi_send_preprocess(4, sizeof(ntp));
			stat++;
		}
		break;

	  case 3:
		if (res == WIFI_RES_PROMPT){	// >
			memset(&ntp, 0, sizeof(ntp));
			ntp.control_word = htonl((1<<27)|(3<<24));	// network order
			wifi_send_data((uint8_t *)&ntp, sizeof(ntp));
			stat++;
		}
		break;

	  case 4:
		if (res == WIFI_RES_SEND_OK){
			stat++;
		}
		break;

	  case 5:
		if (check_sock(SOCK_NTP) >= sizeof(ntp)){
			uint32_t	tm;
			uint32_t	wt;
			char	msg[32];
			int		i;
			uint8_t	*p;
			struct RTC_TIME	rtc;

			p = (uint8_t *)&ntp;
			for (i = 0; i < sizeof(ntp); i++){
				*p++ = read_sock(SOCK_NTP);
			}
			
			tm = ntohl(ntp.transmit_timestamp_seconds) + 1;
			//tm -= 1356966000;	// 2013/01/01 00:00:00
			//tm -= 2208988800;	// 1970/01/01 - 1900/01/01
			tm -= (1356966000 + 2208988800);
			wt = ntohl(ntp.transmit_timestamp_fractions);
			wt = (0 - wt);
			wt /= 4294968;
			if (wt > 50){
				wait_ms(wt - 50);
			}
			seccount_to_rtctime(tm, &rtc);
			if (rtc.year >= 15){		// 2015年以降
				adjust_time(tm);		// RTC設定
				ssprintf(msg, "NTP:20%02d/%02d/%02d %02d:%02d:%02d\n",
						 rtc.year, rtc.month, rtc.day,
						 rtc.hour, rtc.min, rtc.sec);
				cputs(msg);
			}

			send_wifi_command("AT+CIPCLOSE=4\r\n", 15);
			//stat++;
			stat = WIFI_STAT_NTP + 6;
			timeout = SecCount + 1;	// 念の為
		}
		break;

	  case 6:
		if (res == WIFI_RES_OK){
			stat = WIFI_STAT_SLEEP;		// SLEEP
		}
		break;
	}

	if (SecCount > timeout){
		write_log_msg("NTP Timeout\n");
		stat = WIFI_STAT_WAIT;
	} else if (res == WIFI_RES_ERROR){
		write_log_msg("NTP ERROR\n");
		stat = WIFI_STAT_WAIT;
	}

	return stat;
}

