//*******************************************************
//
//	NTPclock
//
//	M25PE20 Serial Flash ROM 処理
//
//	2015.9.19	KAWAKAMI Yukio
//*******************************************************

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"hw_config.h"
#include	"main.h"
#include	"config.h"
#include	"port.h"
#include	"com.h"
#include	"rtc.h"
#include	"ssprintf.h"
#include	"spi.h"
#include	"timer.h"
#include	"rom.h"

extern COM_WORK	STDIO_COM;

extern volatile uint16_t	AckTimer;

#define	SPI_out(x)	spi_trx(SPI1, x)
#define	SPI_in()	spi_trx(SPI1, 0xFF)

//*********************************************************
//	M25PE20 ID取得
//*********************************************************
void ROM_id(uint8_t buff[3]){
	int	i;

	MCS_ON;
	SPI_out(0x9F);
	for (i = 0; i < 3; i++){
		buff[i] = SPI_in();
	}
	MCS_OFF;
}


//*********************************************************
//	M25PE20 書き込み終了待ち
//*********************************************************
void ROM_wait(void){

	// 書き込み終了待ち
	while(ROM_status() & 1){
		WDR;
	}
}


//*********************************************************
//	M25PE20 読み込み
//*********************************************************
void ROM_read(uint32_t adrs, uint8_t *buff, uint16_t n){

	MCS_ON;
	SPI_out(0x03);
	SPI_out((uint8_t)(adrs >> 16));
	SPI_out((uint8_t)(adrs >> 8));
	SPI_out((uint8_t)adrs);
	if (n < 32){
		while(n > 0){
			*buff++ = SPI_in();
			--n;
		}
	} else {
		// 転送数が多い場合は DMA使用
		DMA_InitTypeDef		DMA_init;
		uint8_t	dummy = 0xFF;
		uint16_t	cr1;

		RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
		DMA_DeInit(DMA1_Channel2);	// SPI1_RX
		DMA_DeInit(DMA1_Channel3);	// SPI1_TX
		DMA_init.DMA_PeripheralBaseAddr = (uint32_t)&SPI1->DR;
		DMA_init.DMA_BufferSize = n;
		DMA_init.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
		DMA_init.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
		DMA_init.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
		DMA_init.DMA_Mode = DMA_Mode_Normal;
		DMA_init.DMA_Priority = DMA_Priority_High;
		DMA_init.DMA_M2M = DMA_M2M_Disable;

		DMA_init.DMA_MemoryInc = DMA_MemoryInc_Disable;
		DMA_init.DMA_MemoryBaseAddr = (uint32_t)&dummy;
		DMA_init.DMA_DIR = DMA_DIR_PeripheralDST;
		DMA_Init(DMA1_Channel3, &DMA_init);

		DMA_init.DMA_MemoryInc = DMA_MemoryInc_Enable;
		DMA_init.DMA_MemoryBaseAddr = (uint32_t)buff;
		DMA_init.DMA_DIR = DMA_DIR_PeripheralSRC;
		DMA_Init(DMA1_Channel2, &DMA_init);
		
		SPI_I2S_DMACmd(SPI1, SPI_I2S_DMAReq_Tx | SPI_I2S_DMAReq_Rx, ENABLE);

		cr1 = SPI1->CR1;
		SPI1->CR1 &= 0xFFC7;	// BR = 0 で、24MHz/2 = 12MHzクロック

		DMA_Cmd(DMA1_Channel2, ENABLE);
		DMA_Cmd(DMA1_Channel3, ENABLE);

		while(DMA1_Channel2->CNDTR);

		SPI1->CR1 = cr1;
	}
	MCS_OFF;
}


//*********************************************************
//	M25PE20 書き込み(プログラム)
//	ただし、ページ(256bytes)にまたがってはいけない
//*********************************************************
void ROM_write(uint32_t adrs, uint8_t *buff, uint16_t n){

	// 念のため書き込み終了待ち
	ROM_wait();

	MCS_ON;
	SPI_out(0x06);		// 書き込み許可。１回書き込むと自動解除されるらしい
	MCS_OFF;

	MCS_ON;
	SPI_out(0x02);		// Page Program
	SPI_out((uint8_t)(adrs >> 16));
	SPI_out((uint8_t)(adrs >> 8));
	SPI_out((uint8_t)adrs);
	if (n < 32){
		while(n > 0){
			SPI_out(*buff++);
			--n;
		}
	} else {
		// 転送数が多い場合は DMA使用
		DMA_InitTypeDef		DMA_init;
		uint8_t		dummy;
		uint16_t	cr1;

		RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
		DMA_DeInit(DMA1_Channel2);	// SPI1_RX
		DMA_DeInit(DMA1_Channel3);	// SPI1_TX
		DMA_init.DMA_PeripheralBaseAddr = (uint32_t)&SPI1->DR;
		DMA_init.DMA_BufferSize = n;
		DMA_init.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
		DMA_init.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
		DMA_init.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
		DMA_init.DMA_Mode = DMA_Mode_Normal;
		DMA_init.DMA_Priority = DMA_Priority_High;
		DMA_init.DMA_M2M = DMA_M2M_Disable;

		DMA_init.DMA_MemoryInc = DMA_MemoryInc_Enable;
		DMA_init.DMA_MemoryBaseAddr = (uint32_t)buff;
		DMA_init.DMA_DIR = DMA_DIR_PeripheralDST;
		DMA_Init(DMA1_Channel3, &DMA_init);

		DMA_init.DMA_MemoryInc = DMA_MemoryInc_Disable;
		DMA_init.DMA_MemoryBaseAddr = (uint32_t)&dummy;
		DMA_init.DMA_DIR = DMA_DIR_PeripheralSRC;
		DMA_Init(DMA1_Channel2, &DMA_init);
		
		SPI_I2S_DMACmd(SPI1, SPI_I2S_DMAReq_Tx | SPI_I2S_DMAReq_Rx, ENABLE);

		cr1 = SPI1->CR1;
		SPI1->CR1 &= 0xFFC7;	// BR = 0 で、24MHz/2 = 12MHzクロック

		DMA_Cmd(DMA1_Channel2, ENABLE);
		DMA_Cmd(DMA1_Channel3, ENABLE);

		while(DMA1_Channel2->CNDTR);

		SPI1->CR1 = cr1;
	}
	MCS_OFF;

	ROM_wait();	// 書き込み終了待ち
}


//*********************************************************
//	M25PE20 Status Register 読み込み
//*********************************************************
uint8_t ROM_status(void){
	uint8_t	stat;

	MCS_ON;
	SPI_out(0x05);
	stat = SPI_in();
	MCS_OFF;

	return stat;
}


//*********************************************************
//	M25PE20 サブセクタ消去
//	１セクタは 32KB
//*********************************************************
void ROM_sector_erase(uint32_t adrs){

	while(ROM_status() & 1);	// 消去待ち

	MCS_ON;
	SPI_out(0x06);		// 書き込み許可。１回書き込むと自動解除されるらしい
	MCS_OFF;

	//adrs &= ROM_SECTOR_MASK;

	MCS_ON;
	SPI_out(0x20);
	SPI_out((uint8_t)(adrs >> 16));
	SPI_out((uint8_t)(adrs >> 8));
	SPI_out((uint8_t)adrs);
	MCS_OFF;

	while(ROM_status() & 1);	// 消去待ち
}


//*********************************************************
//	M25PE20 ページ消去
//	1ページ 256bytes
//*********************************************************
void ROM_page_erase(uint32_t adrs){

	while(ROM_status() & 1);	// 消去待ち

	MCS_ON;
	SPI_out(0x06);		// 書き込み許可。１回書き込むと自動解除されるらしい
	MCS_OFF;

	adrs &= ROM_PAGE_MASK;

	MCS_ON;
	SPI_out(0xDB);
	SPI_out((uint8_t)(adrs >> 16));
	SPI_out((uint8_t)(adrs >> 8));
	SPI_out((uint8_t)adrs);
	MCS_OFF;

	while(ROM_status() & 1);	// 消去待ち
}


//*********************************************************
//	指定byte数受信
//*********************************************************
int read_block(uint8_t *buff, int len){
	int	n = 0;

	AckTimer = 1000;
	while(AckTimer && (len > 0)){
		while(check_com(&STDIO_COM)){
			*buff++ = read_com(&STDIO_COM);
			--len;
			n++;
		}
	}

	return n;
}


//*********************************************************
//	ACK送信
//*********************************************************
void send_ack(void){

	write_com(&STDIO_COM, ACK);
}


//*********************************************************
//	NACK送信
//*********************************************************
void send_nack(void){

	write_com(&STDIO_COM, NACK);
}


//*********************************************************
//	リソースファイル受信
//*********************************************************
int load_resource(void){
	TRANS_FILE	fdata;
	uint32_t	remain;
	uint32_t	adrs;
	int			rc = 0;

	send_ack();

	if (read_block((uint8_t *)&fdata, sizeof(TRANS_FILE)) != sizeof(TRANS_FILE)){
		send_nack();
		cputs("\nLoad Error\n");
		return 1;
	}

	remain = fdata.fsize;	
	if ((remain == 0)||(remain > RESOURCE_SIZE)){
		char	msg[32];
		send_nack();
		ssprintf(msg, "\nFile Size Error : %d\n", remain);
		cputs(msg);
		return 2;
	} else if (strncmp(fdata.fname, "RESOURCE.DAT", 12) != 0){
		send_nack();
		cputs("\nFile Name Error\n");
		return 3;
	}

	adrs = RESOURCE_ADRS;
	send_ack();

	while(remain > 0){
		uint8_t	buff[256];
		int		len;

		len = (int)remain;
		if (len > sizeof(buff))	len = sizeof(buff);
		if (read_block(buff, len) != len){
			send_nack();
			cputs("\nData Receive Error\n");
			rc = 4;
			break;
		}

		ROM_page_erase(adrs);
		ROM_write(adrs, buff, len);
		adrs += len;
		remain -= len;
		send_ack();
	}

	cputs("\n");

	return rc;
}


//*********************************************************
//	リソースの情報取得
//*********************************************************
void get_resource_data(int n, uint32_t *a, uint32_t *s){

	if ((n < 0)||(n > 255))	return;

	int		i;
	uint32_t	adrs;
	uint32_t	size;
	uint32_t	table;

	if (n == 0)	return;

	adrs = RESOURCE_ADRS + 256 * 4;
	table = RESOURCE_ADRS + 4;
	for (i = 1; i < n; i++){
		ROM_read(table, (uint8_t *)&size, 4);
		adrs += size;
		table += 4;
	}
	ROM_read(table, (uint8_t *)&size, 4);
	if ((size == 0)||(size == 0xFFFFFFFF))	return;

	*a = adrs;
	*s = size;
}


//*********************************************************
//	ROMから改行までの文字列読み込み
//*********************************************************
int resource_gets(int rn, char *str_buff, int maxlen){
	static char		buff[256];
	static int16_t	buff_p = 0;
	static uint32_t	adrs;
	static uint32_t	remain;
	int	len = 0;

	if (rn != 0){
		get_resource_data(rn, &adrs, &remain);
		buff_p = 256;
	}

	--maxlen;

	while(maxlen > 0){
		char	c;
		if (buff_p >= sizeof(buff)){
			int	n;
			if (remain <= 0)	break;
			n = remain;
			if (remain > sizeof(buff))	n = sizeof(buff);
			ROM_read(adrs, (uint8_t *)buff, n);
			adrs += n;
			remain -= n;
			buff_p = 0;
		}
		c = buff[buff_p++];
		*str_buff++ = c;
		*str_buff = 0;
		--maxlen;
		++len;
		if (c == LF)	break;
	}

	return len;
}
