//*******************************************************
//
//	NTPclock
//
//	シリアル端末操作
//
//	2015.11.27	KAWAKAMI Yukio
//*******************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "hw_config.h"
#include "main.h"
#include "config.h"
#include "port.h"
#include "com.h"
#include "rtc.h"
#include "term.h"
#include "program.h"
#include "ssprintf.h"
#include "spi.h"
#include "rom.h"
#include "wifi.h"
#include "log.h"

volatile int	IntCount;

extern uint8_t	Debug;
extern COM_WORK	STDIO_COM;
extern CONFIG	Config;

extern const char *BootMsg;

extern volatile uint16_t	AckTimer;
extern volatile uint32_t	TickCount;
extern volatile uint16_t	RtcIntFlag;
extern struct RTC_TIME		Rtc;	// 現在時刻
extern uint32_t NowSec;		// 現在秒数値


//*******************************************************
//	デバッグ端末操作
//*******************************************************
void job_term(void){
	static uint8_t	com_buff[80];
	static int		com_buff_p = 0;

	while(check_com(&STDIO_COM)){
		u8		c;

		c = read_com(&STDIO_COM);

		if (c == '\r'){	// CR
			command(com_buff);	// コマンド処理
			com_buff[0] = 0;
			com_buff_p = 0;
		} else if (c == 8){		// BS
			if (com_buff_p > 0){
				--com_buff_p;
				com_buff[com_buff_p] = 0;
				cputs("\b \b");
			}
		} else if ((c < ' ')||(c >= 0x7F)){
			;	// 無視
		} else if (com_buff_p < (sizeof(com_buff) - 1)){
			// CR以外の文字入力
			com_buff[com_buff_p++] = c;
			com_buff[com_buff_p] = 0;
		}
	}
}


//*******************************************************
//	コマンド処理
//*******************************************************
void command(uint8_t *com_buff){
	COM_ARG	arg;
	char	msg[32];

	split_arg(com_buff, &arg);	// 引数分割

	if (strcmp(arg.arg0, "VER") == 0){
		cputs(BootMsg);
	} else if (strcmp(arg.arg0, "BOOT") == 0){
		wait_send_com(&STDIO_COM);
		boot();
		cputs("Can't Start Bootloader\n");
	} else if ((strcmp(arg.arg0, "SPM_PAGESIZE") == 0)||
			   (strcmp(arg.arg0, "FLASH_PAGESIZE") == 0)){
		ssprintf(msg, "%d\n", FLASH_PAGE_SIZE);
		cputs(msg);
	} else if (strcmp(arg.arg0, "FIRM") == 0){
		strupr(arg.arg1);
		if (strcmp(arg.arg1, "UPDATE") == 0){
			Config.program_mark = 'M';
			save_config(&Config);
			boot();
		}
	} else if (strcmp(arg.arg0, "INT") == 0){
		ssprintf(msg, "IntCount=%d\n", IntCount);
		cputs(msg);
	} else if (strcmp(arg.arg0, "UPLOAD") == 0){
		if (load_resource() == 0){
			write_log_msg("Load RESOURCE.DAT");
			cputs("OK\n");
		}
	} else if (strcmp(arg.arg0, "DEBUG") == 0){
		int	n;
		n = get_value(arg.arg1);
		if (n){
			Debug = 1;
			Config.flags |= CONFIG_FLAGS_DEBUG;
			save_config(&Config);
		} else {
			Debug = 0;
			Config.flags &= ~CONFIG_FLAGS_DEBUG;
			save_config(&Config);
		}
		cputs("OK\n");
	} else if (strcmp(arg.arg0, "STAT") == 0){
		ssprintf(msg, "WifiStat : %04Xh\n", get_wifi_stat());
		cputs(msg);
	} else if (strcmp(arg.arg0, "SLEEP") == 0){
		int	n;
		n = get_value(arg.arg1);
		set_sleep(n);
		cputs("OK\n");
	} else if (strcmp(arg.arg0, "START") == 0){
		strupr(arg.arg1);
		if (strcmp(arg.arg1, "HTTP") == 0){
		} else if (strcmp(arg.arg1, "NTP") == 0){
			if (start_ntp()){
				cputs("OK\n");
			} else {
				cputs("Not Start NTP\n");
			}
		} else if (strcmp(arg.arg1, "WIFI") == 0){
			start_wifi();
			cputs("OK\n");
		}
	} else if (strcmp(arg.arg0, "CLEAR") == 0){
		strupr(arg.arg1);
		if (strcmp(arg.arg1, "LOG") == 0){
			clear_log();
			write_log("Clear LOG");
			cputs("OK\n");
		}
	} else if (strcmp(arg.arg0, "LOG") == 0){
		strupr(arg.arg1);
		if (strcmp(arg.arg1, "CLEAR") == 0){
			clear_log();
			write_log("Clear LOG");
			cputs("OK\n");
		} else {
			uint32_t	tm;
			tm = get_value(arg.arg1);
			if (tm > 0){	// 未指定は全て出力
				// 何日前からのログデータ出力
				tm = read_RTC_value() - (tm * 60 * 60 * 24);
			}
			print_logs(tm);
			cputs("OK\n");
		}
	} else if (strcmp(arg.arg0, "SSID") == 0){
		if (!arg.arg1[0]){
			cputs(Config.ssid);
			cputs("\n");
		} else {
			if (strlen(arg.arg1) < 32){
				strcpy(Config.ssid, arg.arg1);
				save_config(&Config);
				cputs("OK\n");
			}
		}
	} else if (strcmp(arg.arg0, "PWD") == 0){
		if (!arg.arg1[0]){
			cputs(Config.pwd);
			cputs("\n");
		} else {
			if (strlen(arg.arg1) < 64){
				strcpy(Config.pwd, arg.arg1);
				save_config(&Config);
				cputs("OK\n");
			}
		}
	} else if (strcmp(arg.arg0, "NTP") == 0){
		if (!arg.arg1[0]){
			cputs(Config.ntp_server);
			cputs("\n");
		} else {
			if (strlen(arg.arg1) < 32){
				strcpy(Config.ntp_server, arg.arg1);
				save_config(&Config);
				cputs("OK\n");
			}
		}
	} else if (strcmp(arg.arg0, "IP") == 0){
		if (!arg.arg1[0]){
			ssprintf(msg, "%d.%d.%d.%d\n",
					 Config.my_ip[0],
					 Config.my_ip[1],
					 Config.my_ip[2],
					 Config.my_ip[3]);
			cputs(msg);
		} else {
			int	n1, n2, n3, n4;
			if (sscanf(arg.arg1, "%d.%d.%d.%d", &n1, &n2, &n3, &n4) == 4){
				if ((n1 >= 0)&&(n1 <= 255)&&
					(n2 >= 0)&&(n2 <= 255)&&
					(n3 >= 0)&&(n3 <= 255)&&
					(n4 >= 0)&&(n4 <= 255)){
					Config.my_ip[0] = n1;
					Config.my_ip[1] = n2;
					Config.my_ip[2] = n3;
					Config.my_ip[3] = n4;
					save_config(&Config);
					cputs("OK\n");
				}
			}
		}
	} else if (strcmp(arg.arg0, "PORT") == 0){
		if (!arg.arg1[0]){
			ssprintf(msg, "%d\n", Config.my_port);
			cputs(msg);
		} else {
			int n;
			n = atoi(arg.arg1);
			if ((n >= 0)&&(n <= 65535)){
				Config.my_port = n;
				save_config(&Config);
				cputs("OK\n");
			}
		}
	} else if (strcmp(arg.arg0, "WDR") == 0){
		int	n = 0;
		while(!RtcIntFlag);
		WDR;
		for(;;){
			ssprintf(msg, "%d\n", n++);
			cputs(msg);
			RtcIntFlag = 0;
			while(!RtcIntFlag);
		}
	} else if (strcmp(arg.arg0, "CHIPINFO") == 0){
		uint16_t	*p;
		uint8_t	buff[20];
		int	i;

		p = (uint16_t *)0x1FFFF7E0;
		ssprintf(msg, "Flash Size : %dKB\n", *p);
		cputs(msg);
		p = (uint16_t *)0x1FFFF7E8;
		cputs("UID : ");
		for (i = 0; i < 96 / 16; i++){
			if (i)	cputs(":");
			ssprintf(msg, "%02X", *p & 0x00FF);
			cputs(msg);
			ssprintf(msg, ":%02X", (*p >> 8) & 0x00FF);
			cputs(msg);
			p++;
		}
		cputs("\n");
		ROM_id(buff);
		cputs("Serial ROM ID:\n");
		for (i = 0; i < 20; i++){
			ssprintf(msg, "%02X ", buff[i]);
			cputs(msg);
		}
		cputs("\n");
	} else if (strcmp(arg.arg0, "RTC") == 0){
		strupr(arg.arg1);
		if (!arg.arg1[0]){
			uint32_t	tm, cal;
			struct RTC_TIME	rtc;
			cal = ((int)Config.rtc_cal) - 60;
			if (Config.rtc_cal == 0)	cal = 0;
			ssprintf(msg, "Config.rtc_cal : %+d\n", cal);
			cputs(msg);
			tm = read_backup_ram(BKP_REG_ADJUSTED_TIME_L);
			tm |= ((uint32_t)read_backup_ram(BKP_REG_ADJUSTED_TIME_H)) << 16;
			seccount_to_rtctime(tm, &rtc);
			ssprintf(msg, "last adjusted : 20%02d/%02d/%02d %02d:%02d:%02d\n",
					 rtc.year, rtc.month, rtc.day,
					 rtc.hour, rtc.min, rtc.sec);
			cputs(msg);
		} else if (strcmp(arg.arg1, "CAL") == 0){
			if (arg.arg2[0]){
				int	n;
				n = get_value(arg.arg2);
				if ((n > -120)&&(n <= 120)){
					Config.rtc_cal = n;
					save_config(&Config);
					set_RTC_cal(n);
					cputs("OK\n");
				}
			}
		}
	} else if (strcmp(arg.arg0, "INIT") == 0){
		strupr(arg.arg1);
		if (strcmp(arg.arg1, "CONFIG") == 0){
			init_config(&Config);
			save_config(&Config);
			cputs("OK\n");
		} else if (strcmp(arg.arg1, "RTC") == 0){
			uint32_t	count;
			count = read_RTC_value();
			init_RTC();
			write_RTC_value(count + 1);
			write_backup_ram(BKP_DR1, RTC_BACKUP_MARK);
			cputs("OK\n");
		}
	} else if (strcmp(arg.arg0, "CONFIG") == 0){
		print_config(&Config, cputs);
	} else if (strcmp(arg.arg0, "CLOCK") == 0){
		RCC_ClocksTypeDef RCC_Clocks;
		RCC_GetClocksFreq(&RCC_Clocks);
		ssprintf(msg, "ADC : %ld\n", RCC_Clocks.ADCCLK_Frequency);	cputs(msg);
		ssprintf(msg, "HCLK : %ld\n", RCC_Clocks.HCLK_Frequency);	cputs(msg);
		ssprintf(msg, "PCLK1 : %ld\n", RCC_Clocks.PCLK1_Frequency);	cputs(msg);
		ssprintf(msg, "PCLK2 : %ld\n", RCC_Clocks.PCLK2_Frequency);	cputs(msg);
		ssprintf(msg, "SYSCLK : %ld\n", RCC_Clocks.SYSCLK_Frequency);	cputs(msg);
		ssprintf(msg, "SystemCoreClock : %ld\n", SystemCoreClock);	cputs(msg);
		switch(RCC_GetSYSCLKSource()){
		  case 0x00:
			cputs("HSI");	break;
		  case 0x04:
			cputs("HSE");	break;
		  case 0x08:
			cputs("PLL");	break;
		  default:
			cputs("UnKown");	break;
		}
		cputs(" Clock Source\n");
	} else if (strcmp(arg.arg0, "INIT") == 0){
		strupr(arg.arg1);
		if (strcmp(arg.arg1, "RTC") == 0){
			init_RTC();
			write_backup_ram(BKP_DR1, RTC_BACKUP_MARK);
			cputs("OK\n");
		} else if (strcmp(arg.arg1, "CONFIG") == 0){
			init_config(&Config);
			save_config(&Config);
			cputs("OK\n");
		}
	} else if (strcmp(arg.arg0, "DATE") == 0){
		struct RTC_TIME	rtc;
		if (arg.arg1[0]){
			int	y, m, d;
			if (sscanf(arg.arg1, "%d/%d/%d",
					   &y, &m, &d) == 3){
				uint32_t	tm;
				read_RTC(&rtc);
				if (y > 2000)	y -= 2000;
				rtc.year = y;
				rtc.month = m;
				rtc.day = d;
				tm = get_seccount(&rtc);
				adjust_time(tm);
			}
		} else {
			read_RTC(&rtc);
			ssprintf(msg, "20%d/%02d/%02d %02d:%02d:%02d\n",
					 rtc.year, rtc.month, rtc.day,
					 rtc.hour, rtc.min, rtc.sec);
			cputs(msg);
		}
	} else if (strcmp(arg.arg0, "WATCH") == 0){
		while(!check_com(&STDIO_COM)){
			WDR;
			SLEEP;
			if (RtcIntFlag){
				RtcIntFlag = 0;
				read_RTC(&Rtc);
				ssprintf(msg, "20%d/%02d/%02d %02d:%02d:%02d\r",
						 Rtc.year, Rtc.month, Rtc.day,
						 Rtc.hour, Rtc.min, Rtc.sec);
				cputs(msg);
			}
		}
		read_com(&STDIO_COM);
		cputs("\n");
	} else if (strcmp(arg.arg0, "TIME") == 0){
		struct RTC_TIME	rtc;
		if (arg.arg1[0]){
			int	hh, mm, ss;
			int n;
			if ((n = sscanf(arg.arg1, "%d:%d:%d",
							&hh, &mm, &ss)) >= 2){
				uint32_t	tm;
				read_RTC(&rtc);
				rtc.hour = hh;
				rtc.min = mm;
				if (n == 3){
					rtc.sec = ss;
				} else {
					rtc.sec = 0;
				}
				tm = get_seccount(&rtc);
				adjust_time(tm);
			}
		} else {
			read_RTC(&rtc);
			ssprintf(msg, "20%d/%02d/%02d %02d:%02d:%02d\n",
					 rtc.year, rtc.month, rtc.day,
					 rtc.hour, rtc.min, rtc.sec);
			cputs(msg);
		}
	} else if (strncmp(arg.arg0, "AT", 2) == 0){
		Debug = 1;	// Configには保存しない
		send_wifi_command((char *)com_buff, strlen((char *)com_buff));
		send_wifi_command("\r\n", 2);
	} else if (strncmp(arg.arg0, "+++", 3) == 0){
		Debug = 1;	// Configには保存しない
		send_wifi_command("+++", 3);
	} else if ((strcmp(arg.arg0, "DUMP") == 0)||
			   (strcmp(arg.arg0, "D") == 0)){	// メモリダンプ
		static uint32_t	adrs = APPLICATION_ADDRESS;
		uint8_t	*p, c;
		int	i;
		if (arg.arg1[0] != 0){
			adrs = get_value(arg.arg1);
		}
		p = (uint8_t *)adrs;
		if (adrs < 0x40000000){
			for (i = 0; i < 128; i++){
				if (i == 0){
					uint32_t	a;
					a = adrs & ~15;
					ssprintf(msg, "%08lX:", a);
					cputs(msg);
					while(a < adrs){
						cputs("   ");
						a++;
					}
				} else if((adrs & 15) == 0){
					ssprintf(msg, "%08lX:", adrs);
					cputs(msg);
				}
				c = *p++;
				ssprintf(msg, "%02X ", c);
				cputs(msg);
				if ((++adrs & 15) == 0){
					cputs("\n");
					wait_send_com(&STDIO_COM);
				}
			}
		} else {
			uint32_t	data;
			adrs &= ~3;
			data = *((uint32_t *)adrs);
			ssprintf(msg, "%08lX:%lX", adrs, data);
			cputs(msg);
		}
		cputs("\n");
	}
}


//*******************************************************
//	コマンド文字列を引数に分解
//*******************************************************
void split_arg(uint8_t *buff, COM_ARG *arg){
	char	*p, c;
	int		n;

	memset((void *)arg, 0, sizeof(COM_ARG));
	arg->buff = buff;

	p = (char *)buff;
	n = 0;
	while((c = *p)){
		if ((c == ' ')||(c == '\t'))	break;
		if ((c >= 'a')&&(c <= 'z'))		c &= 0xDF;	// 大文字化
		arg->arg0[n++] = c;
		arg->arg0[n] = 0;
		p++;
		if (n >= (sizeof(arg->arg0)-1)){	// MAX 31文字
			while((c = *p++)){
				if ((c == ' ')||(c == '\t'))	break;
			}
			break;
		}
	}
	while((c = *p)){
		if ((c != ' ')&&(c != '\t'))	break;
		p++;
	}
	if (*p == 0)	return;
	n = 0;
	while((c = *p)){
		if ((c == ' ')||(c == '\t'))	break;
		arg->arg1[n++] = c;
		arg->arg1[n] = 0;
		p++;
		if (n >= (sizeof(arg->arg1)-1)){	// MAX 31文字
			while((c = *p++)){
				if ((c == ' ')||(c == '\t'))	break;
			}
			break;
		}
	}
	while((c = *p)){
		if ((c != ' ')&&(c != '\t'))	break;
		p++;
	}
	if (*p == 0)	return;
	n = 0;
	while((c = *p)){
		if ((c == ' ')||(c == '\t'))	break;
		arg->arg2[n++] = c;
		arg->arg2[n] = 0;
		p++;
		if (n >= (sizeof(arg->arg2)-1)){	// MAX 31文字
			while((c = *p++)){
				if ((c == ' ')||(c == '\t'))	break;
			}
			break;
		}
	}
}


//*******************************************************
//	コマンド文字列の数値を値に変換
//*******************************************************
int get_value(char *str){
	int	n = 0;

	if ((strcmp(str, "ON") == 0)||
		(strcmp(str, "on") == 0)){
		n = 1;
	} else if ((strcmp(str, "OFF") == 0)||
			   (strcmp(str, "off") == 0)){
		;
	} else if ((*str == '$')||
			   ((*str >= 'A')&&(*str <= 'F'))||
			   ((*str >= 'a')&&(*str <= 'f'))||
			   (str[strlen(str)-1] == 'h')){
		if (*str == '$')	str++;
		while(is_hex((u8)*str)){
			n <<= 4;
			n += hex_val((u8)*str++);
		}
	} else if ((*str == '-')||
			   (isdigit((int)*str))){
		if (*str == '-'){
			n = -1;
			str++;
		} else {
			n = 1;
		}
		n *= atoi(str);
	}
	return n;
}


//*******************************************************
//	0-9 A-F a-f 判別
//*******************************************************
char is_hex(uint8_t c){

	if (c < '0')	return 0;
	if (c <= '9')	return 1;
	if (c < 'A')	return 0;
	if (c <= 'F')	return 1;
	if (c < 'a')	return 0;
	if (c <= 'f')	return 1;
	return 0;
}


//*******************************************************
//	16進数文字の数値変換
//*******************************************************
uint8_t hex_val(uint8_t c){

	if (c <= '9')	return c - '0';
	if (c <= 'F')	return c - 'A' + 10;
	if (c <= 'f')	return c - 'a' + 10;
	return 0;
}


//*******************************************************
//	空白文字スキップ
//*******************************************************
char *skip_space(char *p){
	char	c;

	while((c = *p)){
		if ((c != ' ')&&(c != '\t'))	break;
		p++;
	}
	return p;
}


//*******************************************************
//	次の文字列先頭までスキップ
//*******************************************************
char *skip_next_word(char *p){
	char	c;

	while((c = *p)){
		if ((c == ' ')||(c == '\t'))	break;
		p++;
	}
	return skip_space(p);
}


