//*******************************************************
//
//	NTPclock
//
//	タイマー処理
//
//	2015.11.17	KAWAKAMI Yukio
//*******************************************************

#include "hw_config.h"

#include "main.h"
#include "port.h"
#include "config.h"
#include "timer.h"

volatile uint16_t	WTimer;
volatile uint16_t	AckTimer;
volatile uint16_t	WifiTimer;

volatile uint32_t	TickCount = 0;

extern volatile uint8_t		RtcIntFlag;
extern volatile uint16_t	RtcDelay;

//************************************************************
//	1ms単位の経過時間取得
//************************************************************
uint32_t tick(uint32_t *last_tick){
	uint32_t	now_tick;
	uint32_t	ret;

	now_tick = TickCount;
	if (now_tick < *last_tick){
		ret = now_tick + (0xFFFFFFFF - *last_tick) + 1;
	} else {
		ret = now_tick - *last_tick;
	}
	*last_tick = now_tick;
	return ret;
}


//************************************************************
//	SysTick interrupt
//	1kHzで呼び出す
//************************************************************
void SysTick_Handler(void){

	if (RtcDelay){	// RTC割り込みは、秒が変わる 1/32768秒前に起きる
		if (--RtcDelay == 0){	// ので、少し遅らせてからフラグを立てる
			RtcIntFlag = 1;
		}
	}

	if (WTimer)		--WTimer;
	if (AckTimer)	--AckTimer;
	if (WifiTimer)	--WifiTimer;

	TickCount++;
}


//*******************************************************
//	タイマー初期設定
//*******************************************************
void init_timer(void){
	RCC_ClocksTypeDef RCC_Clocks;

	// SysTick設定 1kHz
	RCC_GetClocksFreq(&RCC_Clocks);
	SysTick_Config(RCC_Clocks.SYSCLK_Frequency / 8 / 1000);
	SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK_Div8);
}


//*******************************************************
//	1μs単位で時間待ち
//*******************************************************
void wait_us(int tm){
	RCC_ClocksTypeDef RCC_Clocks;
	RCC_GetClocksFreq(&RCC_Clocks);

	RCC->APB1ENR |= RCC_APB1Periph_TIM4;

	TIM4->PSC = RCC_Clocks.PCLK1_Frequency / 1000000 - 1;
	TIM4->ARR = tm;
	TIM4->CNT = 0;
	TIM4->EGR = TIM_EGR_UG;
	TIM4->CR1 |= (TIM_CR1_OPM | TIM_CR1_CEN);
	while (TIM4->CR1 & TIM_CR1_CEN);

	RCC->APB1ENR &= ~RCC_APB1Periph_TIM4;
}


//*******************************************************
//	1ms単位で時間待ち
//*******************************************************
void wait_ms(int tm){

	WTimer = tm;
	while(WTimer){
		WDR;
		SLEEP;
	}
}


