//*******************************************************
//
//	NTPclock
//
//	Wifi / ESP-WROOM-02 Module
//
//	2015.10.12	KAWAKAMI Yukio
//*******************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "hw_config.h"
#include "main.h"
#include "config.h"
#include "port.h"
#include "com.h"
#include "program.h"
#include "ssprintf.h"
#include "spi.h"
#include "wifi.h"
#include "rtc.h"
#include "log.h"
#include "rom.h"

extern COM_WORK	STDIO_COM;
extern COM_WORK	WIFI_COM;
extern CONFIG	Config;
extern uint8_t	Debug;

extern uint32_t	SecCount;	// 起動後の経過秒数
extern struct RTC_TIME	Rtc;
extern uint32_t NowSec;		// 現在秒数値

extern uint16_t	WifiTimer;

uint16_t	WifiStat;
static uint32_t	StartTime;

uint8_t		SockBuff[SOCK_CH_N][1024];
uint16_t	SockBuff_in[SOCK_CH_N], SockBuff_out[SOCK_CH_N], SockBuff_remain[SOCK_CH_N];
uint8_t		SockConnect;
uint8_t		IpdCh;
uint8_t		HttpSock = 0xFF;

uint8_t		WaitSendPrompt;



//*******************************************************
//	Job Wifi Module
//*******************************************************
void job_wifi(void){
	static uint8_t	buff[80];
	static uint8_t	buff_p = 0;
	static uint16_t	ipd = 0;
	static uint8_t	ich = 0;
	char	res_flag = 0;
	int	disconnect_sock = -1;

	while(check_com(&WIFI_COM)){
		int	c;
		res_flag = 0;
		c = read_com(&WIFI_COM);
		if (ipd > 0){
			SockBuff[ich][SockBuff_in[ich]++] = c;
			if (SockBuff_in[ich] >= SOCK_BUFF_SIZE)	SockBuff_in[ich] = 0;
			SockBuff_remain[ich]++;
			if (--ipd == 0)		break;	// 一旦抜ける
			if (SockBuff_remain[ich] == SOCK_BUFF_SIZE)	break;
		} else if (c == CR){
			;
		} else if (c == LF){
			if (strcmp((char *)buff, "OK") == 0){
				res_flag = WIFI_RES_OK;
			} else if (strcmp((char *)buff, "SEND OK") == 0){
				res_flag = WIFI_RES_SEND_OK;
			} else if (strcmp((char *)buff, "ready") == 0){
				res_flag = WIFI_RES_READY;
			} else if (strcmp((char *)buff, "ERROR") == 0){
				res_flag = WIFI_RES_ERROR;
			} else if (strcmp((char *)&buff[1], ",CONNECT") == 0){
				int	ch;
				ch = buff[0] - '0';
				if ((ch >= 0)&&(ch < SOCK_CH_N)){
					SockConnect |= (1 << ch);
					SockBuff_in[ch] = SockBuff_out[ch] = SockBuff_remain[ch] = 0;
					if (ch == SOCK_NTP){
						res_flag = WIFI_RES_NTP_CONNECT;
					} else if (HttpSock == 0xFF){
						res_flag = WIFI_RES_SOCK_CONNECT;
					} else {
						res_flag = WIFI_RES_LF;
					}
				}
			} else if (strcmp((char *)&buff[1], ",CLOSED") == 0){
				int	ch;
				ch = buff[0] - '0';
				if ((ch >= 0)&&(ch < SOCK_CH_N)){
					SockConnect &= ~(1 << ch);
					if (ch == SOCK_NTP){
						res_flag = WIFI_RES_NTP_CLOSED;
					} else if (ch == HttpSock){
						disconnect_sock = ch;
						res_flag = WIFI_RES_SOCK_CLOSED;
						HttpSock = 0xFF;
					} else {
						res_flag = WIFI_RES_LF;
					}
				}
			} else if (strcmp((char *)buff, "WIFI GOT IP") == 0){
				res_flag = WIFI_RES_WIFI_CONNECT;
			} else if (strcmp((char *)buff, "WIFI DISCONNECT") == 0){
				res_flag = WIFI_RES_WIFI_DISCONNECT;
			} else if (buff_p == 0){
				res_flag = WIFI_RES_LF_NODATA;
			} else if (strncmp((char *)buff, "+CIPAP:", 7) == 0){
				if (!Debug){
					cputs((char *)&buff[7]);
					cputs("\n");
				}
				res_flag = WIFI_RES_LF;
			} else {
				res_flag = WIFI_RES_LF;
			}
			if (Debug && buff[0]){
				cputs((char *)buff);
				cputs("\n");
			}
			break;	// 一旦抜ける
		} else if (buff_p < (sizeof(buff)-1)){
			if ((c != ' ')||(buff_p > 0)){	// 行頭にスペースが入ることがある
				buff[buff_p++] = c;
				buff[buff_p] = 0;
			}
			if ((WaitSendPrompt)&&(c == '>')){
				WaitSendPrompt = 0;
				res_flag = WIFI_RES_PROMPT;
				buff[0] = 0;
				buff_p = 0;
				break;	// 一旦抜ける
			} else if ((c == ':')&&
				(strncmp((char *)buff, "+IPD,", 5) == 0)){
				ipd = atoi((char *)&buff[7]);
				if (Debug){
					cputs((char *)buff);	cputs("\n");
				}
				buff[0] = 0;
				buff_p = 0;
				ich = buff[5] - '0';
				if (ich >= SOCK_CH_N)	ich = 0;
			}
		}
	}

	if (res_flag == WIFI_RES_SOCK_CLOSED){
		char	msg[32];
		ssprintf(msg, "Sock %d Disconnect", disconnect_sock);
		write_log_msg(msg);
		WifiStat = WIFI_STAT_DISCONNECT;
	}

	if ((!is_sock_connect(SOCK_NTP))&&
		(HttpSock == 0xFF)){
		int		i, sock;
		sock = disconnect_sock + 1;
		for (i = 0; i < SOCK_NTP; i++){
			if (sock >= SOCK_NTP)	sock = 0;
			if (SockConnect & (1 << sock)){
				char	msg[32];
				HttpSock = sock;
				WifiStat = WIFI_STAT_HTTP;
				ssprintf(msg, "Sock %d Connect", sock);
				write_log_msg(msg);
				ipd = 0;
				break;
			}
			sock++;
		}
	}

	switch(WifiStat & 0xF000){
	  case WIFI_STAT_NON:		// Wifi接続開始
		WifiStat = wifi_connect(WifiStat, res_flag);
		break;
	  case WIFI_STAT_NTP:		// NTP処理
		WifiStat = wifi_ntp(WifiStat, res_flag);
		break;
	  case WIFI_STAT_START:		// HTTPサーバ起動
		break;
	  case WIFI_STAT_WAIT:		// Sock接続待ち
		break;
	  case WIFI_STAT_HTTP:		// Sock接続・HTTP開始
		break;
	  case WIFI_STAT_DISCONNECT:	// Sock切断
		if ((WifiStat & 0x00FF) == 0){
			WifiStat = WIFI_STAT_WAIT;
		}
		break;
	  case WIFI_STAT_CLOSE:		// サーバclose
		break;
	  case WIFI_STAT_SLEEP:		// SLEEP
		WifiStat = wifi_sleep(WifiStat, res_flag);
		break;
	  case WIFI_STAT_ERROR:		// ERROR
		break;
	}

	if (Debug){
		static uint16_t	last_stat = 0;
		if (WifiStat != last_stat){
			char	msg[32];
			ssprintf(msg, "WifiStat : %04Xh\n", WifiStat);
			cputs(msg);
			last_stat = WifiStat;
		}
	}
	
	if (res_flag >= WIFI_RES_LF){
		buff[0] = 0;
		buff_p = 0;
	}
}


//*******************************************************
//	Wifi接続処理
//*******************************************************
uint16_t wifi_connect(uint16_t stat, int res){
	static uint32_t	timeout;
	uint32_t	tm;

	switch(stat & 0x00FF){
	  case 0:
		tm = SecCount;
		timeout = SecCount + 15;
		res = 0;
		if (tm > (StartTime + 10)){	// 起動後10秒
			RST_ON;
			WifiTimer = 20;	// 20ms
			stat++;
		}
		break;
	  case 1:
		if (!WifiTimer){
			RST_OFF;
			WifiTimer = 50;	// 50ms
			stat++;
		}
		break;
	  case 2:
		if (!WifiTimer){
			send_wifi_command("AT+RST\r\n", 8);
			stat++;
		}
		break;
	  case 3:
		if (res == WIFI_RES_READY){
			send_wifi_command("ATE0\r\n", 6);
			stat++;
		}
		break;
	  case 4:
		if (res == WIFI_RES_OK){
			send_wifi_command("AT+SLEEP=2\r\n", 12);
			stat++;
		}
		break;
	  case 5:
		if (res == WIFI_RES_OK){
			send_wifi_command("AT+CWMODE=1\r\n", 13);
			stat++;
		}
		break;
	  case 6:
		if (res == WIFI_RES_OK){
			send_wifi_command("AT+CWDHCP=1,1\r\n", 15);
			stat++;
		}
		break;
	  case 7:
		if (res == WIFI_RES_OK){
			send_wifi_command("AT+CWAUTOCONN=0\r\n", 17);
			stat++;
		}
		break;
	  case 8:
		if ((res == WIFI_RES_OK)||
			(res == WIFI_RES_ERROR)){	// AT+CWAUTOCONNは新しいコマンド
			send_wifi_command("AT+CWJAP=\"", 10);
			send_wifi_command(Config.ssid, strlen(Config.ssid));
			send_wifi_command("\",\"", 3);
			send_wifi_command(Config.pwd, strlen(Config.pwd));
			send_wifi_command("\"\r\n", 3);
			if (Debug)	cputs("***\",\"***\"\r\n");
			stat++;
		}
		break;
	  case 9:
		if (res == WIFI_RES_WIFI_CONNECT){
			write_log_msg("Wifi Connect");
		} else if (res == WIFI_RES_OK){
			stat = WIFI_STAT_NTP;	// NTP開始
		}
		break;
	}

	if (SecCount > timeout){
		write_log_msg("Wifi Timeout\n");
		stat = WIFI_STAT_ERROR;
	} else if (res == WIFI_RES_ERROR){
		write_log_msg("Wifi ERROR\n");
		stat = WIFI_STAT_ERROR;
	}

	return stat;
}


//*******************************************************
//	SLEEP処理
//*******************************************************
uint16_t wifi_sleep(uint16_t stat, int res){
	static uint32_t	timeout;

	switch(stat & 0x00FF){
	  case 0:
		timeout = SecCount + 5;
		stat++;
		break;
	  case 1:
		send_wifi_command("AT+GSLP=0\r\n", 11);	// deep-sleep mode
		stat++;
		break;
	  case 2:
		if (res == WIFI_RES_OK){
			stat++;
		}
		break;
	  case 3:
		return stat;
	}

	if (SecCount > timeout){
		write_log_msg("Sleep Timeout\n");
		stat = WIFI_STAT_ERROR;
	} else if (res == WIFI_RES_ERROR){
		write_log_msg("Sleep ERROR\n");
		stat = WIFI_STAT_ERROR;
	}
	return stat;
}


//*******************************************************
//	WifiStat 取得
//*******************************************************
uint16_t get_wifi_stat(void){
	return WifiStat;
}


//*******************************************************
//	Wifi接続開始
//*******************************************************
void start_wifi(void){

	if (((WifiStat & 0xF000) == WIFI_STAT_ERROR)||
		((WifiStat & 0xF000) == WIFI_STAT_SLEEP)){
		WifiStat = WIFI_STAT_NON;
	}
}


//*******************************************************
//	Wifiモジュール SLEEP ON/OFF
//*******************************************************
void set_sleep(int flag){

	if (flag){
		send_wifi_command("AT+SLEEP=1\r\n", 12);
	} else {
		RST_ON;
		wait_ms(20);
		RST_OFF;
		wait_ms(100);
		send_wifi_command("AT+SLEEP=2\r\n", 12);
	}
}


//*******************************************************
//	Sock データがあるか
//*******************************************************
int check_sock(int ch){

	return SockBuff_remain[ch];
}


//*******************************************************
//	Sock 接続中かどうか
//*******************************************************
int is_sock_connect(int ch){

	if (SockConnect & (1 << ch)){
		return 1;
	} else {
		return 0;
	}
}


//*******************************************************
//	Sockデータ読み込み
//*******************************************************
int read_sock(int ch){
	int	c;

	if (SockBuff_remain[ch] == 0)	return 0;

	c = SockBuff[ch][SockBuff_out[ch]++];
	if (SockBuff_out[ch] >= SOCK_BUFF_SIZE)	SockBuff_out[ch] = 0;
	--SockBuff_remain[ch];

	return c;
}


//*******************************************************
//	Init Wifi Module
//*******************************************************
void init_wifi(void){

	WifiStat = WIFI_STAT_NON;
	StartTime = SecCount;

	RST_OFF;

	init_com(&WIFI_COM, USART2, 115200);
}


//*******************************************************
//	Wifi コマンド送信
//*******************************************************
void send_wifi_command(const char *data, int len){
	int	flag = 0;

	if ((data[0] == 'A')&&(data[1] == 'T'))	flag = Debug;

	while(len > 0){
		if (flag)	write_com(&STDIO_COM, *data);

		write_com(&WIFI_COM, *data++);
		--len;
	}
}


//*******************************************************
//	TCP, UDP データ送信開始前処理
//*******************************************************
void wifi_send_preprocess(int sock, int len){
	char	msg[32];

	ssprintf(msg, "AT+CIPSEND=%d,%d\r\n", sock, len);
	send_wifi_command(msg, strlen(msg));
	WaitSendPrompt = 1;	// プロンプト待ち
}


//*******************************************************
//	TCP, UDP データ送信
//*******************************************************
void wifi_send_data(uint8_t *data, int len){
	
	while(len > 0){
		write_com(&WIFI_COM, *data++);
		--len;
	}
}
