//*******************************************************
//
//	NTPclock
//
//	Bootloader
//
//	2015.9.20	KAWAKAMI Yukio
//*******************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "hw_config.h"

#include "main.h"
#include "port.h"
#include "config.h"
#include "com.h"
#include "program.h"
#include "ssprintf.h"
#include "term.h"
#include "timer.h"

CONFIG	Config;

extern COM_WORK		STDIO_COM;
extern volatile uint16_t	WTimer;

int ProgramFlag = 0;

const char *BootMsg = "NTPclock BOOTLOADER Version " BOOT_VERSION "\n"
	"Build Date : " __DATE__ " " __TIME__ "\n";


/*******************************************************************************
* Function Name  : main.
* Description    : Main routine.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
int main(void){

	__disable_irq();	// 割り込み禁止

	SCB->VTOR = NVIC_VectTab_FLASH | (BOOTLOADER_ADDRESS & 0x1FFFFF80);

	init_ports();

	load_config(&Config);

	if (Config.program_mark == 0xFF){	// 書き込み正常終了
		char	*p;
		// RAM末尾のブートマーク確認
		p = (char *)(RAM_ADDRESS + RAM_SIZE - 4);
		if (strncmp(p, "BOOT", 4) != 0){
			boot();		// アプリケーションブート
		}
		*p = 0;
	}

	if (Config.program_mark == 'M'){
		ProgramFlag = 1;	// microSDのファームウェアプログラム
	}

	__enable_irq();

	wait_ms(100);

	LED_ON;

	init_com(&STDIO_COM, USART1, 115200);
	cputs(BootMsg);

	for(;;){
		WDR;
		SLEEP;

		if (!WTimer){
			WTimer = 1000;
			if (IS_LED_ON){
				LED_OFF;
			} else {
				LED_ON;
			}
		}

		job_term();
	}
}


//*******************************************************
//	1秒ごとに割り込み実行する処理
//*******************************************************
void job_sec_intr(void){
}


//********************************************************
//	アプリケーションプログラム ブート
//********************************************************
void boot(void){
	typedef void (*func)(void);
	func	jump_to_app;
	u32		*jump_adrs;
	u32		*p = (u32 *)APPLICATION_ADDRESS;
	u32		sp;

	sp = *p++;
	jump_adrs = (u32 *)(*p);
	jump_to_app = (func)jump_adrs;

	if (((u32)jump_adrs > (u32)APPLICATION_ADDRESS)&&
		((u32)jump_adrs < (u32)(APPLICATION_ADDRESS + FLASH_SIZE))&&
		(sp > RAM_ADDRESS)&&
		(sp <= (RAM_ADDRESS + RAM_SIZE))){
		int	i;

		__disable_irq();	// 割り込み禁止

		USART1->SR = 0;
		USART1->CR1 = 0;
		ADC1->SR = 0;
		ADC1->CR1 = 0;
		SysTick->CTRL = 0;

		for (i = 0; i < 8; i++){
			NVIC->ICER[i] = NVIC->IABR[i];
		}

		EXTI->IMR = 0x00000000;
		EXTI->EMR = 0x00000000;
		EXTI->RTSR = 0x00000000; 
		EXTI->FTSR = 0x00000000; 
		EXTI->PR = 0x000FFFFF;
		RCC_DeInit();

		SCB->VTOR = NVIC_VectTab_FLASH | (APPLICATION_ADDRESS & 0x1FFFFF80);

		__set_MSP(sp);
		jump_to_app();
	}
}



//*******************************************************
//	GPIOポート初期化
//*******************************************************
void init_ports(void){
	GPIO_InitTypeDef GPIO_InitStructure;
	//RCC_ClocksTypeDef RCC_Clocks;
	//ADC_InitTypeDef ADC_InitStructure;
	//NVIC_InitTypeDef NVIC_InitStructure;
	//TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;

	// PWRとBKPクロック供給	SLEEPモードに必要らしい
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
	//RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
	//RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE);

	// JTAGピンを GPIOにリマップ。これ以降、JTAG I/Fは無効
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);
	GPIO_PinRemapConfig(GPIO_Remap_SWJ_Disable, ENABLE);
	BKP_TamperPinCmd(DISABLE);	// タンパーピン無効

	// GPIO初期値
	LED_OFF;
	MCS_OFF;
	RST_OFF;
	SEG_OUT(0);

	// GPIO出力設定
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Pin = PIN_M_CS | PIN_LED;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;	// ノイズを避ける
	GPIO_InitStructure.GPIO_Pin = 0x00FF;	// 7SEG アノード側
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	// GPIO オープンドレイン
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_OD;
	GPIO_InitStructure.GPIO_Pin = PIN_RST;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = 0xFF00;	// 7SEG カソード側
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	// GPIO入力プルアップ
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_InitStructure.GPIO_Pin = PIN_WUP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// GPIO入力プルダウン
	//GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;

	// GPIO入力
	//GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;;
	
	init_timer();
}


#ifdef USE_FULL_ASSERT
/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert_param error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert_param error line source number
* Output         : None
* Return         : None
*******************************************************************************/
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {}
}
#endif
