//*******************************************************
//
//	NTPclock
//
//	セルフプログラミング
//
//	2015.9.19	KAWAKAMI Yukio
//*******************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "hw_config.h"

#include "main.h"
#include "port.h"
#include "com.h"
#include "program.h"
#include "config.h"
#include "term.h"
#include "ssprintf.h"

#define	PROGRAM_BUFF_SIZE	256

extern CONFIG	Config;
extern COM_WORK	STDIO_COM;

extern volatile uint16_t	AckTimer;

//********************************************************
//	メッセージ表示
//********************************************************
void print_msg(char *msg){

	cputs(msg);
}


//*******************************************************
//	ブロック受信
//*******************************************************
int term_block_read(uint8_t *buff, uint16_t len, uint16_t tm){

	AckTimer = tm;

	while(AckTimer){
		if (check_com(&STDIO_COM)){
			*buff++ = read_com(&STDIO_COM);
			if (--len == 0){
				return 0;
			}
		}
	}
	return 1;
}


//********************************************************
//	シリアルポート読み込み・ファームウェアプログラム
//********************************************************
int update_firm_com(void){
	TRANS_FILE	file;
	uint32_t	size;
	int32_t		remain;
	uint32_t	end_adrs;
	uint32_t	adrs;
	uint8_t		buff[PROGRAM_BUFF_SIZE];
	int			i;
	int			ret = 0;
	uint32_t	*p;

	write_com(&STDIO_COM, ACK);
	
	// ファイル情報読み込み
	if (term_block_read((uint8_t *)&file, sizeof(file), 100)){
		return 0;
	}
	size = file.fsize;	// サイズ
	if ((size < 1024)||(size > FLASH_SIZE)){
		write_com(&STDIO_COM, NACK);
		print_msg("File size Error\n");
		return 1;
	}
	if (strcmp(file.fname, FIRMWARE_FNAME) != 0){
		write_com(&STDIO_COM, NACK);	// ファイル名が異なる
		print_msg("*File name Error\n");
		return 1;
	}

	end_adrs = size + APPLICATION_ADDRESS;
	end_adrs += PROGRAM_BUFF_SIZE - 1;
	end_adrs &= ~(PROGRAM_BUFF_SIZE - 1);	// 終了アドレスを FLASH_PAGE_SIZEに揃える

	// 書き込み中マーク
	load_config(&Config);
	if (Config.program_mark == 0xFF){
		Config.program_mark = 'W';
		save_config(&Config);
	}

	FLASH_Unlock();
	FLASH_ClearFlag(FLASH_FLAG_BSY | FLASH_FLAG_EOP | FLASH_FLAG_PGERR | FLASH_FLAG_WRPRTERR);

	//cls();
	//print("Program");

	adrs = APPLICATION_ADDRESS;
	remain = size;
	while(remain > 0){
		int	l;

		WDR;

		// アドレス表示
		//ssprintf(msg, "%lX ", adrs);
		//locate(0, 1);
		//print(msg);

		if ((adrs & (FLASH_PAGE_SIZE - 1)) == 0){
			// ページ消去
			if (FLASH_ErasePage(adrs) != FLASH_COMPLETE){
				print_msg("*Flash Erase Error\r\n");
				ret = 1;
				break;
			}
		}

		write_com(&STDIO_COM, ACK);

		if (remain >= PROGRAM_BUFF_SIZE){
			l = PROGRAM_BUFF_SIZE;
		} else {
			l = remain;
		}
		if (term_block_read(buff, l, 1000)){
			print_msg("*Time Out Error\n");
			ret = 1;
			break;
		}

		p = (uint32_t *)buff;
		for (i = 0; i < (PROGRAM_BUFF_SIZE / sizeof(u32)); i++){
			if (FLASH_ProgramWord(adrs + i * sizeof(u32), *p)
				!= FLASH_COMPLETE){

				print_msg("*Flash Write Error\r\n");
				adrs = 0xFFFFFFFF;
				ret = 1;
				break;
			}
			p++;
		}
		adrs += PROGRAM_BUFF_SIZE;
		remain -= l;
	}

	FLASH_Lock();

	write_com(&STDIO_COM, ACK);

	wait_send_com(&STDIO_COM);

	if (ret == 0){
		Config.program_mark = 0xFF;		// 書き換え完了
		save_config(&Config);
		print_msg("\r\nOK\r\n");
	} else {
		// 書き変え成功していなければ実行するのは危ないのでクリアする
		wait_ms(10);	// 念のため待つ
		FLASH_Unlock();
		FLASH_ClearFlag(FLASH_FLAG_BSY | FLASH_FLAG_EOP | FLASH_FLAG_PGERR | FLASH_FLAG_WRPRTERR);
		FLASH_ErasePage(APPLICATION_ADDRESS);
		FLASH_Lock();
	}

	return ret;
}


