//*******************************************************
//
//	NTPclock
//
//	Bootloader
//
//	シリアル端末操作
//
//	2015.9.19	KAWAKAMI Yukio
//*******************************************************

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"hw_config.h"
#include	"main.h"
#include	"port.h"
#include	"com.h"
#include	"ssprintf.h"
#include	"program.h"
#include	"term.h"

extern COM_WORK	STDIO_COM;
extern const char *BootMsg;

extern int ProgramFlag;

int read_batt(void);



//*******************************************************
//	デバッグ端末操作
//*******************************************************
void job_term(void){
	static uint8_t	com_buff[80];
	static int		com_buff_p = 0;

	while(check_com(&STDIO_COM)){
		u8		c;
		COM_ARG	arg;
		char	msg[64];

		c = read_com(&STDIO_COM);

		if (c == '\r'){	// CR

			split_arg(com_buff, &arg);	// 引数分割
			if (strcmp(arg.arg0, "VER") == 0){
				cputs(BootMsg);
			} else if (strcmp(arg.arg0, "PROGRAM") == 0){
				strupr(arg.arg1);
				if (strcmp(arg.arg1, "MICROSD") == 0){
					ProgramFlag = 1;
				}
			} else if (strcmp(arg.arg0, "INIT") == 0){
			} else if ((strcmp(arg.arg0, "BOOT") == 0)||
					   (strcmp(arg.arg0, "START") == 0)){
				wait_send_com(&STDIO_COM);
				RCC->CSR |= (1<<24);	// リセットフラグクリア
				boot();
				cputs("Can't Start Application\n");
			} else if ((strcmp(arg.arg0, "SPM_PAGESIZE") == 0)||
					   (strcmp(arg.arg0, "FLASH_PAGESIZE") == 0)){
				ssprintf(msg, "%d\n", FLASH_PAGE_SIZE);
				cputs(msg);
			} else if (strcmp(arg.arg0, "UPLOAD") == 0){
				update_firm_com();
				wait_ms(100);
			} else if (strcmp(arg.arg0, "CLOCK") == 0){
				RCC_ClocksTypeDef RCC_Clocks;
				RCC_GetClocksFreq(&RCC_Clocks);
				ssprintf(msg, "            ADC : %ld\n", RCC_Clocks.ADCCLK_Frequency);
				cputs(msg);
				ssprintf(msg, "           HCLK : %ld\n", RCC_Clocks.HCLK_Frequency);
				cputs(msg);
				ssprintf(msg, "          PCLK1 : %ld\n", RCC_Clocks.PCLK1_Frequency);
				cputs(msg);
				ssprintf(msg, "          PCLK2 : %ld\n", RCC_Clocks.PCLK2_Frequency);
				cputs(msg);
				ssprintf(msg, "         SYSCLK : %ld\n", RCC_Clocks.SYSCLK_Frequency);
				cputs(msg);
				ssprintf(msg, "SystemCoreClock : %ld\n", SystemCoreClock);
				cputs(msg);
			} else if ((strcmp(arg.arg0, "DUMP") == 0)||
					   (strcmp(arg.arg0, "D") == 0)){	// メモリダンプ
				static uint32_t	adrs = BOOTLOADER_ADDRESS;
				uint8_t	*p, c;
				int	i;
				if (arg.arg1[0] != 0){
					adrs = get_value(arg.arg1);
				}
				p = (uint8_t *)adrs;
				if (adrs < 0x40000000){
					for (i = 0; i < 128; i++){
						if (i == 0){
							uint32_t	a;
							a = adrs & ~15;
							ssprintf(msg, "%08lX:", a);
							cputs(msg);
							while(a < adrs){
								cputs("   ");
								a++;
							}
						} else if((adrs & 15) == 0){
							ssprintf(msg, "%08lX:", adrs);
							cputs(msg);
						}
						c = *p++;
						ssprintf(msg, "%02X ", c);
						cputs(msg);
						if ((++adrs & 15) == 0){
							cputs("\n");
							wait_send_com(&STDIO_COM);
						}
					}
				} else {
					uint32_t	data;
					adrs &= ~3;
					data = *((uint32_t *)adrs);
					ssprintf(msg, "%08lX:%lX", adrs, data);
					cputs(msg);
				}
				cputs("\n");
			}
			com_buff[0] = 0;
			com_buff_p = 0;
		} else if (c == 8){		// BS
			if (com_buff_p > 0){
				--com_buff_p;
				com_buff[com_buff_p] = 0;
				cputs("\b \b");
			}
		} else if ((c < ' ')||(c >= 0x7F)){
			;	// 無視
		} else if (com_buff_p < (sizeof(com_buff) - 1)){
			// CR以外の文字入力
			com_buff[com_buff_p++] = c;
			com_buff[com_buff_p] = 0;
		}
	}
	
}


//*******************************************************
//	コマンド文字列を引数に分解
//*******************************************************
void split_arg(uint8_t *buff, COM_ARG *arg){
	char	*p, c;
	int		n;

	memset((void *)arg, 0, sizeof(COM_ARG));
	arg->buff = buff;

	p = (char *)buff;
	n = 0;
	while((c = *p)){
		if ((c == ' ')||(c == '\t'))	break;
		if ((c >= 'a')&&(c <= 'z'))		c &= 0xDF;	// 大文字化
		arg->arg0[n++] = c;
		arg->arg0[n] = 0;
		p++;
		if (n >= (sizeof(arg->arg0)-1)){	// MAX 31文字
			while((c = *p++)){
				if ((c == ' ')||(c == '\t'))	break;
			}
			break;
		}
	}
	while((c = *p)){
		if ((c != ' ')&&(c != '\t'))	break;
		p++;
	}
	if (*p == 0)	return;
	n = 0;
	while((c = *p)){
		if ((c == ' ')||(c == '\t'))	break;
		arg->arg1[n++] = c;
		arg->arg1[n] = 0;
		p++;
		if (n >= (sizeof(arg->arg1)-1)){	// MAX 31文字
			while((c = *p++)){
				if ((c == ' ')||(c == '\t'))	break;
			}
			break;
		}
	}
	while((c = *p)){
		if ((c != ' ')&&(c != '\t'))	break;
		p++;
	}
	if (*p == 0)	return;
	n = 0;
	while((c = *p)){
		if ((c == ' ')||(c == '\t'))	break;
		arg->arg2[n++] = c;
		arg->arg2[n] = 0;
		p++;
		if (n >= (sizeof(arg->arg2)-1)){	// MAX 31文字
			while((c = *p++)){
				if ((c == ' ')||(c == '\t'))	break;
			}
			break;
		}
	}
}


//*******************************************************
//	コマンド文字列の数値を値に変換
//*******************************************************
int get_value(char *str){
	int	n = 0;

	if ((strcmp(str, "ON") == 0)||
		(strcmp(str, "on") == 0)){
		n = 1;
	} else if ((strcmp(str, "OFF") == 0)||
			   (strcmp(str, "off") == 0)){
		;
	} else if ((*str == '$')||
			   ((*str >= 'A')&&(*str <= 'F'))||
			   ((*str >= 'a')&&(*str <= 'f'))||
			   (str[strlen(str)-1] == 'h')){
		if (*str == '$')	str++;
		while(is_hex((u8)*str)){
			n <<= 4;
			n += hex_val((u8)*str++);
		}
	} else if ((*str == '-')||
			   (isdigit((int)*str))){
		if (*str == '-'){
			n = -1;
			str++;
		} else {
			n = 1;
		}
		n *= atoi(str);
	}
	return n;
}


//*******************************************************
//	0-9 A-F a-f 判別
//*******************************************************
char is_hex(uint8_t c){

	if (c < '0')	return 0;
	if (c <= '9')	return 1;
	if (c < 'A')	return 0;
	if (c <= 'F')	return 1;
	if (c < 'a')	return 0;
	if (c <= 'f')	return 1;
	return 0;
}


//*******************************************************
//	16進数文字の数値変換
//*******************************************************
uint8_t hex_val(uint8_t c){

	if (c <= '9')	return c - '0';
	if (c <= 'F')	return c - 'A' + 10;
	if (c <= 'f')	return c - 'a' + 10;
	return 0;
}


//*******************************************************
//	空白文字スキップ
//*******************************************************
char *skip_space(char *p){
	char	c;

	while((c = *p)){
		if ((c != ' ')&&(c != '\t'))	break;
		p++;
	}
	return p;
}


//*******************************************************
//	次の文字列先頭までスキップ
//*******************************************************
char *skip_next_word(char *p){
	char	c;

	while((c = *p)){
		if ((c == ' ')||(c == '\t'))	break;
		p++;
	}
	return skip_space(p);
}


