//*******************************************************
//
//	NTPclock
//
//	タイマー処理
//
//	2015.11.15	KAWAKAMI Yukio
//*******************************************************

#include "hw_config.h"

#include "main.h"
#include "port.h"
#include "config.h"
#include "timer.h"

volatile uint16_t	WTimer;
volatile uint16_t	AckTimer;

volatile uint32_t	TickCount = 0;


//************************************************************
//	SysTick interrupt
//	1kHzで呼び出す
//************************************************************
void SysTick_Handler(void){

	if (WTimer)		--WTimer;
	if (AckTimer)	--AckTimer;

	TickCount++;
}


//*******************************************************
//	タイマー初期設定
//*******************************************************
void init_timer(void){
	RCC_ClocksTypeDef RCC_Clocks;

	// SysTick設定 1kHz
	RCC_GetClocksFreq(&RCC_Clocks);
	SysTick_Config(RCC_Clocks.SYSCLK_Frequency / 8 / 1000);
	SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK_Div8);
}


//*******************************************************
//	1μs単位で時間待ち
//*******************************************************
void wait_us(int tm){

	RCC->APB1ENR |= RCC_APB1Periph_TIM4;

	TIM4->PSC = SystemCoreClock/1000000 - 1;
	TIM4->ARR = tm;
	TIM4->CNT = 0;
	TIM4->EGR = TIM_EGR_UG;
	TIM4->CR1 |= (TIM_CR1_OPM | TIM_CR1_CEN);
	while (TIM4->CR1 & TIM_CR1_CEN);

	RCC->APB1ENR &= ~RCC_APB1Periph_TIM4;
}


//*******************************************************
//	1ms単位で時間待ち
//*******************************************************
void wait_ms(int tm){

	WTimer = tm;
	while(WTimer){
		WDR;
		SLEEP;
	}
}




