//*******************************************************
//
//	NTPclock
//
//	シリアルポート制御
//
//	2015.9.19	KAWAKAMI Yukio
//*******************************************************

#include	"hw_config.h"

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"main.h"
#include	"com.h"

COM_WORK	Com1, Com2, Com3;

COM_WORK	*USART1_Com;
COM_WORK	*USART2_Com;
COM_WORK	*USART3_Com;

//*******************************************************
//	上位レベル関数
//*******************************************************
void putch(uint8_t data){

	write_com(&STDIO_COM, data);
}


uint8_t getch(void){

	if (check_com(&STDIO_COM)){
		return read_com(&STDIO_COM);
	} else {
		return 0;
	}
}

uint8_t keypressed(void){

	if (check_com(&STDIO_COM)){
		return 1;
	} else {
		return 0;
	}
}


void cputs(const char *s){
	char	c;

	while((c = *s++)){
		if (c == '\n'){
			putch('\r');
		}
		putch(c);
	}
}


void cgets(char *s, int bufsize){
	char *p;
	int c;

	memset(s, 0, bufsize);

	p = s;

	for (p = s; p < s + bufsize-1;){
		/* 20090521Nemui */
		do{		
			c = getch();
		} while(c == false);
		/* 20090521Nemui */
		switch (c)
			{
			  case '\r' :
			  case '\n' :
				putch('\r');
				putch('\n');
				*p = '\n';
				return;

			  case '\b' :
				if (p > s)
					{
						*p-- = 0;
						putch('\b');
						putch(' ');
						putch('\b');
					}
				break;

			  default :
				putch(c);
				*p++ = c;
				break;
			}
	}

	return;
}


//*******************************************************
//	シリアルポート初期化
//*******************************************************
void init_com(COM_WORK *com, USART_TypeDef *usart, int baud){
	GPIO_InitTypeDef	gpio;
	USART_InitTypeDef	usart_init;
	NVIC_InitTypeDef NVIC_InitStructure;

	gpio.GPIO_Speed = GPIO_Speed_50MHz;

	if (usart == USART1){
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);
		gpio.GPIO_Pin = GPIO_Pin_9;
		gpio.GPIO_Mode = GPIO_Mode_AF_PP;
		GPIO_Init(GPIOA, &gpio);
		gpio.GPIO_Pin = GPIO_Pin_10;
		gpio.GPIO_Mode = GPIO_Mode_IPU;
		GPIO_Init(GPIOA, &gpio);
		NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
		USART1_Com = com;
	} else if (usart == USART2){
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);
		gpio.GPIO_Pin = GPIO_Pin_2;
		gpio.GPIO_Mode = GPIO_Mode_AF_PP;
		GPIO_Init(GPIOA, &gpio);
		gpio.GPIO_Pin = GPIO_Pin_3;
		gpio.GPIO_Mode = GPIO_Mode_IPU;
		GPIO_Init(GPIOA, &gpio);
		NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQn;
		USART2_Com = com;
	} else if (usart == USART3){
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);
		gpio.GPIO_Pin = GPIO_Pin_10;
		gpio.GPIO_Mode = GPIO_Mode_AF_PP;
		GPIO_Init(GPIOB, &gpio);
		gpio.GPIO_Pin = GPIO_Pin_11;
		gpio.GPIO_Mode = GPIO_Mode_IPU;
		GPIO_Init(GPIOB, &gpio);
		NVIC_InitStructure.NVIC_IRQChannel = USART3_IRQn;
		USART3_Com = com;
	} else {
		return;
	}

	memset((void *)com, 0, sizeof(COM_WORK));
	com->usart = usart;

	USART_DeInit(usart);

	usart_init.USART_BaudRate = baud;
	usart_init.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	usart_init.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	usart_init.USART_Parity = USART_Parity_No;
	usart_init.USART_WordLength = USART_WordLength_8b;
	usart_init.USART_StopBits = USART_StopBits_1;
	USART_Init(usart, &usart_init);

	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	USART_ITConfig(usart, USART_IT_RXNE, ENABLE);

	USART_Cmd(usart, ENABLE);
}


//*******************************************************
//	送受信バッファクリア
//*******************************************************
void clear_buff_com(COM_WORK *com){

	com->usart->CR1 &= ~USART_CR1_RXNEIE;	// 受信割り込み禁止
	com->usart->CR1 &= ~USART_CR1_TXEIE;	// 送信割り込み禁止
	com->ibuff_in = com->ibuff_out = com->ibuff_remain = 0;
#if TX_BUFF_SIZE > 0
	com->obuff_in = com->obuff_out = com->obuff_remain = 0;
#endif
	com->usart->CR1 |= USART_CR1_TXEIE;		// 送信割り込み許可
	com->usart->CR1 |= USART_CR1_RXNEIE;	// 受信割り込み許可
}


//*******************************************************
//	シリアルデータ送信完了まで待つ
//*******************************************************
void wait_send_com(COM_WORK *com){

	// 送信バッファが空になるまで待つ
	while(com->obuff_remain)	SLEEP;
	// 更に送信完了まで待つ
	while(!(com->usart->SR & USART_SR_TC))	NOP;
}


//*******************************************************
//	シリアルポート受信バッファチェック
//*******************************************************
int check_com(COM_WORK *com){

	return com->ibuff_remain;
}


//*******************************************************
//	シリアルポートデータ受信
//*******************************************************
int read_com(COM_WORK *com){
	int	data;

	// 受信データが無ければ来るまで待つ
	while(!com->ibuff_remain){
		WDR;
		SLEEP;
	}
	
	// 受信バッファからデータ読み出し
	data = com->ibuff[com->ibuff_out++];
	if (com->ibuff_out >= sizeof(com->ibuff)){
		com->ibuff_out = 0;
	}
	com->usart->CR1 &= ~USART_CR1_RXNEIE;	// 受信割り込み禁止
	com->usart->CR1 &= ~USART_CR1_TXEIE;	// 送信割り込み禁止
	--com->ibuff_remain;
	com->usart->CR1 |= USART_CR1_TXEIE;		// 送信割り込み許可
	com->usart->CR1 |= USART_CR1_RXNEIE;	// 受信割り込み許可

	return data;
}


//*******************************************************
//	シリアルポートデータ送信
//*******************************************************
void write_com(COM_WORK *com, u8 data){

	// 送信バッファがいっぱいなら待つ
	while(com->obuff_remain >= TX_BUFF_SIZE){
		WDR;
		SLEEP;
	}

	com->usart->CR1 &= ~USART_CR1_RXNEIE;	// 受信割り込み禁止
	com->usart->CR1 &= ~USART_CR1_TXEIE;	// 送信割り込み禁止
	if ((com->obuff_remain == 0)&&
		(com->usart->SR & USART_SR_TXE)){
		com->usart->DR = data;
	} else {
		// データを送信バッファに積む
		com->obuff[com->obuff_in++] = data;
		if (com->obuff_in >= sizeof(com->obuff)){
			com->obuff_in = 0;
		}
		com->obuff_remain++;
	}
	com->usart->CR1 |= USART_CR1_TXEIE;		// 送信割り込み許可
	com->usart->CR1 |= USART_CR1_RXNEIE;	// 受信割り込み許可
}


//*******************************************************
//	シリアルポート文字列送信
//*******************************************************
void write_com_str(COM_WORK *com, const char *str){
	char	c;

	while((c = *str++)){
		write_com(com, (u8)c);
	}
}


//*******************************************************
//	USART1 割り込みハンドラ
//*******************************************************
void USART1_IRQHandler(void){
	u16	sr;

	sr = USART1->SR;
	
	if (sr & (USART_SR_PE | USART_SR_FE | USART_SR_NE | USART_SR_ORE)){
		u8	data = data;
		data = USART1->DR;	// エラークリアの為の読み出し
		return;
	}

	// 受信割り込み
	if (sr & USART_SR_RXNE){
		USART1_Com->ibuff[USART1_Com->ibuff_in++] = USART1->DR;
		if (USART1_Com->ibuff_in >= RX_BUFF_SIZE){
			USART1_Com->ibuff_in = 0;
		}
		(USART1_Com->ibuff_remain)++;
	}

	// 送信割り込み
	if (sr & USART_SR_TXE){
		if (USART1_Com->obuff_remain){
			USART1->DR = USART1_Com->obuff[USART1_Com->obuff_out++];
			if (USART1_Com->obuff_out >= TX_BUFF_SIZE){
				USART1_Com->obuff_out = 0;
			}
			--USART1_Com->obuff_remain;
		}
		if (USART1_Com->obuff_remain == 0){
			USART1->CR1 &= ~USART_CR1_TXEIE;	// 送信割り込み禁止
		}
	}
}


//*******************************************************
//	USART2 割り込みハンドラ
//*******************************************************
void USART2_IRQHandler(void){
	u16	sr;

	sr = USART2->SR;

	if (sr & (USART_SR_PE | USART_SR_FE | USART_SR_NE | USART_SR_ORE)){
		u8	data = data;
		data = USART2->DR;	// エラークリアの為の読み出し
		return;
	}

	// 受信割り込み
	if (sr & USART_SR_RXNE){
		USART2_Com->ibuff[USART2_Com->ibuff_in++] = USART2->DR;
		if (USART2_Com->ibuff_in >= RX_BUFF_SIZE){
			USART2_Com->ibuff_in = 0;
		}
		(USART2_Com->ibuff_remain)++;
	}

	// 送信割り込み
	if (sr & USART_SR_TXE){
		if (USART2_Com->obuff_remain){
			USART2->DR = USART2_Com->obuff[USART2_Com->obuff_out++];
			if (USART2_Com->obuff_out >= TX_BUFF_SIZE){
				USART2_Com->obuff_out = 0;
			}
			--USART2_Com->obuff_remain;
		}
		if (USART2_Com->obuff_remain == 0){
			USART2->CR1 &= ~USART_CR1_TXEIE;	// 送信割り込み禁止
		}
	}
}


//*******************************************************
//	USART3 割り込みハンドラ
//*******************************************************
void USART3_IRQHandler(void){
	u16	sr;

	sr = USART3->SR;

	if (sr & (USART_SR_PE | USART_SR_FE | USART_SR_NE | USART_SR_ORE)){
		u8	data = data;
		data = USART3->DR;	// エラークリアの為の読み出し
		return;
	}

	// 受信割り込み
	if (sr & USART_SR_RXNE){
		USART3_Com->ibuff[USART3_Com->ibuff_in++] = USART3->DR;
		if (USART3_Com->ibuff_in >= RX_BUFF_SIZE){
			USART3_Com->ibuff_in = 0;
		}
		(USART3_Com->ibuff_remain)++;
	}

	// 送信割り込み
	if (sr & USART_SR_TXE){
		if (USART3_Com->obuff_remain){
			USART3->DR = USART3_Com->obuff[USART3_Com->obuff_out++];
			if (USART3_Com->obuff_out >= TX_BUFF_SIZE){
				USART3_Com->obuff_out = 0;
			}
			--USART3_Com->obuff_remain;
		}
		if (USART3_Com->obuff_remain == 0){
			USART3->CR1 &= ~USART_CR1_TXEIE;	// 送信割り込み禁止
		}
	}
}

