//*******************************************************
//
//	NTPclock
//
//	CONFIG, セルフプログラミング
//
//	2015.9.19	KAWAKAMI Yukio
//*******************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "hw_config.h"
#include "main.h"
#include "port.h"
#include "config.h"
#include "com.h"
#include "ssprintf.h"


//***************************************************************************
//	CONFIG内容を出力
//***************************************************************************
void print_config(CONFIG *config, void (*out_func)(const char *msg)){
	char	msg[80];

	ssprintf(msg, "rtc_cal = %d\n", config->rtc_cal);	out_func(msg);
	ssprintf(msg, "flags = %04Xh\n", config->flags);	out_func(msg);
	ssprintf(msg, "my_ip = %d.%d.%d.%d\n",
			 config->my_ip[0],
			 config->my_ip[1],
			 config->my_ip[2],
			 config->my_ip[3]);		out_func(msg);
	ssprintf(msg, "my_port = %d\n", config->my_port);	out_func(msg);
	ssprintf(msg, "ssid = %s\n", config->ssid);	out_func(msg);
	ssprintf(msg, "pwd = %s\n", config->pwd);	out_func(msg);
	ssprintf(msg, "ntp_server = %s\n", config->ntp_server);	out_func(msg);
}


//***************************************************************************
//	CONFIGデータ初期化
//***************************************************************************
void init_config(CONFIG *config){

	memset((void *)config, 0, sizeof(CONFIG));

	config->program_mark = 0xFF;
	config->flags = 0;
	config->rtc_cal = 0;
	config->my_ip[0] = 0;
	config->my_ip[1] = 0;
	config->my_ip[2] = 0;
	config->my_ip[3] = 0;
	config->my_port = 80;
	strcpy(config->ssid, "SSID");
	strcpy(config->pwd, "passwd");
	strcpy(config->ntp_server, "ntp.nict.jp");
	config->mark = CONFIG_MARK;
}


//***************************************************************************
//	RAM上へ CONFIGデータ読み出し
//***************************************************************************
void load_config(CONFIG *config){

	memcpy((void *)config, (void *)FLASH_CONFIG_ADDRESS, sizeof(CONFIG));
}


//***************************************************************************
//	RAM上の CONFIGデータ書き込み
//***************************************************************************
void save_config(CONFIG *config){
	u8	buff[FLASH_PAGE_SIZE];

	memcpy((void *)buff, (void *)CONFIG_PAGE, FLASH_PAGE_SIZE);
	memcpy((void *)&buff[FLASH_PAGE_SIZE - CONFIG_MAX_SIZE],
		   (void *)config, sizeof(CONFIG));

	write_flash(CONFIG_PAGE, buff);
}


//***************************************************************************
//	FLASHデータ１ページ書き込み
//***************************************************************************
void write_flash(u32 adrs, u8 *buff){
	u32	*data;
	int	i;

	FLASH_Unlock();
	FLASH_ClearFlag(FLASH_FLAG_BSY | FLASH_FLAG_EOP | FLASH_FLAG_PGERR | FLASH_FLAG_WRPRTERR);
	FLASH_ErasePage(adrs);
	data = (u32 *)buff;
	for (i = 0; i < FLASH_PAGE_SIZE/sizeof(u32); i++){
		FLASH_ProgramWord(adrs, *data++);
		adrs += sizeof(u32);
	}
	FLASH_Lock();
}
