//*******************************************************
//
//	NTPclock
//
//	RTC
//
//	2015.12.4	KAWAKAMI Yukio
//*******************************************************

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"hw_config.h"
#include	"main.h"
#include	"config.h"
#include	"port.h"
#include	"rtc.h"

extern CONFIG	Config;

struct RTC_TIME	Rtc;

volatile uint8_t	RtcIntFlag;
volatile uint8_t	RtcAlarmFlag;
volatile uint32_t	RtcCount;

volatile uint16_t	RtcDelay;
volatile uint16_t	RtcDelayCounter;


const char MonthDays[]
	= { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

void SetSysClock(void);		// system_stm32f10x.c

//************************************************************
//	RTC interrupt
//	秒が変わる直前に呼び出されることに注意
//************************************************************
void RTC_IRQHandler(void){

	if (RTC->CRL & RTC_IT_SEC){
		RTC->CRL &= ~RTC_IT_SEC;
		//RTC_WaitForLastTask();
		//RtcIntFlag = 1;
		RtcDelay = 2;	// 32kHzの1クロック後に秒が変わるのでそれから作業開始
		job_sec_intr();
	}
}


//************************************************************
//	RTC Alarm interrupt
//************************************************************
void RTCAlarm_IRQHandler(void){

	if (RTC->CRL & RTC_IT_ALR){
		EXTI_ClearITPendingBit(EXTI_Line17);
		if (PWR_GetFlagStatus(PWR_FLAG_WU) != RESET){
			PWR_ClearFlag(PWR_FLAG_WU);
		}
		RTC_WaitForLastTask();
		RTC->CRL &= ~RTC_IT_ALR;
		RTC_WaitForLastTask();
		RtcAlarmFlag = 1;
	}
}


//*******************************************************
//	RTC 割り込み設定
//*******************************************************
void set_RTC_int(void){
	NVIC_InitTypeDef NVIC_InitStructure;
	EXTI_InitTypeDef EXTI_InitStructure;

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);

	// RTC 1秒割り込み
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);
	NVIC_InitStructure.NVIC_IRQChannel = RTC_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 5;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	RTC_ITConfig(RTC_IT_SEC | RTC_IT_ALR, ENABLE);

	// RTC EXTI割り込み STOPモードから起きる為に必要
	EXTI_ClearITPendingBit(EXTI_Line17);
	EXTI_InitStructure.EXTI_Line = EXTI_Line17;
	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
	EXTI_Init(&EXTI_InitStructure);

	// RTC Alarm割り込み
	NVIC_InitStructure.NVIC_IRQChannel = RTCAlarm_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 4;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
}



//********************************************************
//	STOPモードへ入る
//	RTCで 59秒になるか、あるいは ESC or ENTERボタンを押したら抜ける
//	1秒ごとに int func(void) を実行する
//********************************************************
void stop_to_59(int (*func)(void)){

	for(;;){
		enter_stop(0);
		if (func != NULL){
			if (func())	break;
		}
		//if (IS_SW_ENTER || IS_SW_ESC)	break;
		read_RTC(&Rtc);
		if (Rtc.sec == 59)	break;
	}
}


//********************************************************
//	STOPモードへ入る
//	STOPモード中は割り込みも止まる
//	tm秒後に復帰して戻る
//	tmは 2秒まで。それ以上だと WDリセットがかかる
//********************************************************
void enter_stop(int tm){

	PWR_BackupAccessCmd(ENABLE);
	RTC_WaitForLastTask();
	RTC_SetAlarm(RTC_GetCounter() + tm);
	RTC_WaitForLastTask();

	//IWDG->KR = 0x5555;
	//rlr = IWDG->RLR;
	//IWDG->RLR = 0x0FFF;
	//WDR;
	//IWDG->KR = 0xCCCC;

	LED_OFF;
	WDR;
	PWR_EnterSTOPMode(PWR_Regulator_ON, PWR_STOPEntry_WFI);	// ここで停止
	WDR;

	//SYSCLKConfig_STOP();
	SetSysClock();	// system_stm32f10x.c

	//IWDG->KR = 0x5555;
	//IWDG->PR = 0x06;		// 1/256
	//IWDG->RLR = 480;		// ウォッチドッグ 約3秒
	//IWDG->KR = 0xAAAA;
	//IWDG->KR = 0xCCCC;		// enable

	LED_ON;
	//PWR_BackupAccessCmd(DISABLE);
}


const uint16_t	BackupRegs[] = {
	BKP_REG_RUNNING_TIME_L,
	BKP_REG_RUNNING_TIME_H,
	BKP_REG_BATTERY_TIME_L,
	BKP_REG_BATTERY_TIME_H
};


//*******************************************************
//	RTCキャリブレーション設定
//	1/32768 = 30ppm
//	1/(60*60*24) = 11.6ppm
//	rtc_calは+1で約1ppm遅れる
//	121 >= cal >= -121
//*******************************************************
void set_RTC_cal(int cal){

	if (cal > 121)	cal = 121;
	if (cal < -121)	cal = -121;

	PWR_BackupAccessCmd(ENABLE);
	if (cal >= 0){
		RTC_SetPrescaler(32768 - 1);
		BKP_SetRTCCalibrationValue(cal);
	} else {
		RTC_SetPrescaler(32768 - 1 - 4);	// 120ppm程度速くなる
		BKP_SetRTCCalibrationValue(cal + 121);
	}
}


//*******************************************************
//	RTC初期化
//*******************************************************
void init_RTC(void){
	uint16_t	backup[sizeof(BackupRegs)/sizeof(uint16_t)];
	int	i;

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);
	PWR_BackupAccessCmd(ENABLE);

	// 稼働時間データは初期化せずに後で書き戻す
	for (i = 0; i < sizeof(BackupRegs)/sizeof(uint16_t); i++){
		backup[i] = BKP_ReadBackupRegister(BackupRegs[i]);
	}
	
	BKP_DeInit();
	BKP_TamperPinCmd(DISABLE);

	//RTC_EnterConfigMode();
	RCC_LSEConfig(RCC_LSE_ON);
	while (RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET){
		wait_ms(10);
	}
	RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);
	RCC_RTCCLKCmd(ENABLE);
	RTC_WaitForSynchro();
	RTC_WaitForLastTask();

	set_RTC_cal(Config.rtc_cal);

	RTC_WaitForLastTask();
	//RTC_ExitConfigMode();

	RTC_SetCounter(9 * 3600);	// 2013/01/01 09:00
	RTC_WaitForLastTask();

	// 稼働時間データ書き戻し
	for (i = 0; i < sizeof(BackupRegs)/sizeof(uint16_t); i++){
		write_backup_ram(BackupRegs[i], backup[i]);
	}
}


//********************************************************
//	バックアップRAM書き込み
//********************************************************
void write_backup_ram(uint16_t adrs, uint16_t data){

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);
	PWR_BackupAccessCmd(ENABLE);
	BKP_ClearFlag();
	BKP_WriteBackupRegister(adrs, data);
	//PWR_BackupAccessCmd(DISABLE);	// DISABLE後も読み込みはできるらしい
}


//********************************************************
//	バックアップRAM読み込み
//********************************************************
uint16_t read_backup_ram(uint16_t adrs){

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);
	return BKP_ReadBackupRegister(adrs);
}


//*******************************************************
//	RTC_TIMEに現在時刻読み込み
//*******************************************************
void read_RTC(struct RTC_TIME *rtc){

	RtcCount = read_RTC_value();
	seccount_to_rtctime(RtcCount, rtc);
}


//*******************************************************
//	RTC 32bit値読み込み
//*******************************************************
uint32_t read_RTC_value(void){

	//PWR_BackupAccessCmd(ENABLE);
	//RTC_WaitForSynchro();
	return RTC_GetCounter();
}


//*******************************************************
//	RTC 32bit値書き込み
//*******************************************************
void write_RTC_value(uint32_t data){

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);
	WDR;
	PWR_BackupAccessCmd(ENABLE);

	RTC_WaitForLastTask();
	RTC_SetCounter(data);
	RTC_WaitForLastTask();

	write_backup_ram(BKP_REG_ADJUSTED_TIME_L, (uint16_t)data);
	write_backup_ram(BKP_REG_ADJUSTED_TIME_H, (uint16_t)(data >> 16));
}


//*******************************************************
//	RTC_TIMEから秒カウント値を取得
//*******************************************************
uint32_t get_seccount(struct RTC_TIME *rtc){
	uint32_t	count = 0;
	int			m;
	int			days;

	if ((rtc == NULL)||(rtc->year < 13))	return 0;

	days = 365 * (rtc->year - 13);	// 2013.1.1 00:00:00 = 0
	days += (rtc->year - 13) / 4;
	for (m = 1; m < rtc->month; m++){
		days += (int)MonthDays[m];
		if ((m == 2)&&((rtc->year % 4) == 0)){
			days++;
		}
	}
	days += (rtc->day - 1);
	count = days * 24 * 60 * 60;
	count += (uint32_t)rtc->hour * 60 * 60;
	count += (uint32_t)rtc->min * 60;
	count += rtc->sec;

	return count;
}


//*******************************************************
//	秒カウント値を RTC_TIMEに変換
//*******************************************************
void seccount_to_rtctime(uint32_t sec, struct RTC_TIME *rtc){
	int		m;
	int		days;

	memset((void *)rtc, 0, sizeof(struct RTC_TIME));

	days = sec / (24 * 60 * 60);
	sec = sec % (24 * 60 * 60);

	days *= 4;
	days += 3;
	rtc->year = (days / (365 * 4 + 1)) + 13;
	days %= (365 * 4 + 1);
	days /= 4;
	
	for (m = 1; m <= 12; m++){
		int	mdays;
		mdays = MonthDays[m];
		if ((m == 2)&&((rtc->year % 4) == 0))	mdays++;
		if (days < mdays)	break;
		days -= mdays;
	}
	rtc->month = m;
	rtc->day = days + 1;

	rtc->sec = sec % 60;
	sec /= 60;
	rtc->min = sec % 60;
	sec /= 60;
	rtc->hour = sec % 24;

	set_week(rtc);
}


//*******************************************************
//	曜日計算をして rtc.weekにセット
//*******************************************************
void set_week(struct RTC_TIME *rtc){
	long	y, m, d, days;

	y = rtc->year + 2000;
	m = rtc->month;
	d = rtc->day;

	if (m < 3){
		--y;
		m += 12;
	}
	days = 365 * y + y / 4 - y / 100 + y / 400 + 306 * (m + 1) / 10 + d - 428;
	rtc->week = days % 7;
}

