//*******************************************************
//
//	sprintfのサブセット
//
//	2013.12.23	KAWAKAMI Yukio
//*******************************************************

#include	<stdlib.h>
#include	<string.h>
#include	<stdarg.h>
#include	<ctype.h>

#include	"ssprintf.h"

static const unsigned char HexChar[16] = {
	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
	'a', 'b', 'c', 'd', 'e', 'f'
};


//********************************************************
//	10進文字列変換
//********************************************************
void dec_str(long n, char *msg){
	char	buff[16];
	int		i;
	char	minus_flag = 0;

	if (n < 0){
		minus_flag = 1;
		n *= -1;
	}

	i = 0;
	do {
		buff[i++] = HexChar[n % 10];
		n /= 10;
	} while(n);

	if (minus_flag){
		*msg++ = '-';
	}
	while(--i >= 0){
		*msg++ = buff[i];
	}
	*msg = 0;
}


//********************************************************
//	16進文字列変換
//********************************************************
void hex_str(long n, char *msg){
	char	buff[16];
	int		i;
	unsigned long	ul;

	ul = (unsigned long)n;

	i = 0;
	do {
		buff[i++] = HexChar[ul & 0x0000000F];
		ul >>= 4;
	} while(ul);

	while(--i >= 0){
		*msg++ = buff[i];
	}
	*msg = 0;
}


//*******************************************************
//	sprintfのサブセット
//*******************************************************
void ssprintf(char *out, const char *form, ...){
	va_list args;
	char	c;
	char	plus_flag;
	char	minus_flag;
	char	long_flag;
	char	out_flag;
	char	padding;
	char	keta;
	char	buff[16];
	char	*out_p;
	long	l;
	char	*format;

	va_start(args, form);
	format = (char *)form;

	while((c = *format++)){
		if (c == '%'){
			plus_flag = 0;
			minus_flag = 0;
			long_flag = 0;
			out_flag = 0;
			padding = ' ';
			keta = 0;
			buff[0] = 0;
			out_p = NULL;
			c = *format;
			if (c == 0)	break;
			while((c = *format++)){
				switch(c){
				  case 'd':
				  case 'x':
				  case 'X':
					if (long_flag){
						l = (long)va_arg(args, long);
					} else {
						l = (long)va_arg(args, int);
					}
					if (c == 'd'){
						if (plus_flag &&(l > 0)){
							buff[0] = '+';
							dec_str(l, &buff[1]);
						} else {
							dec_str(l, buff);
						}
					} else {
						hex_str(l, buff);
						if (c == 'X'){
							strupr(buff);
						}
					}
					out_p = buff;
					out_flag = 1;
					break;
				  case 's':
					out_p = va_arg(args, char *);
					out_flag = 1;
					break;
				  case 'c':
					*out++ = va_arg(args, int);
					out_flag = 1;
					break;
				  case '%':
					*out++ = '%';
					out_flag = 1;
					break;
				  case 'l':
					long_flag = 1;
					break;
				  case '-':
					minus_flag = 1;
					break;
				  case '+':
					plus_flag = 1;
					break;
				  default:
					if (isdigit((int)c)){
						if ((c == '0')&&(keta == 0))	padding = '0';
						keta *= 10;
						keta += c - '0';
					}
					break;
				}
				if (out_flag){
					if (out_p != NULL){
						unsigned char	i, n, pad;
						n = strlen(out_p);
						if (keta == 0)	keta = n;
						if (n > keta)	n = keta;	// 表示桁数優先
						pad = keta - n;
						if ((pad > 0)&&(padding == '0')&&(*out_p == '+')){
							*out++ = '+';
							--n;
							out_p++;
						}
						if (!minus_flag){
							for (i = 0; i < pad; i++)	*out++ = padding;
						}
						for (i = 0; i < n; i++)	*out++ = *out_p++;
						if ((padding == ' ')&& minus_flag){
							for (i = 0; i < pad; i++)	*out++ = padding;
						}
					}
					break;
				}
			}
		} else {
			*out++ = c;
		}
	}
	*out = 0;

	va_end(args);
}
